# build_baseline.R
#
# builds the set of exogenous variables that help define the baseline in sage.
# the output is stored in data/baseline.gms by default



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","download_file.R"))
source(file.path("utilities","install_R_packages.R"))

library(readxl)
library(gdxrrw)
library(matrixStats)


# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# the exogenous current acount and deficit forecasts will linearly decline to
# zero between the end of the cbo forecast and decline_end_year
decline_end_year = 2070

# labor productivity and population growth linearly decline between start_year
# and end_year
start_year = 2060
end_year = 2080

# the variance in sector specific labor productivity growth rates convergences
# to zero linearly between years converge_start and converge_end
converge_start = 2050
converge_end = 2070

# directory where data files will be stored
data_dir = file.path("build","data")

# directory where baseline file is generated
out_dir = file.path("data")

# gams input file to create
output.file = file.path(out_dir,"baseline.gms")

# the benchmark year is the first year in the model
benchmark_year = sage.years[1]

# when aeo data is used these define the aeo release year and scenario to use
aeo.year = 2018
aeo.scenario = paste0("REF",aeo.year)

data.versions = NULL



# ------------------------------------------------------------------------------
# write model years to the baseline
# ------------------------------------------------------------------------------

# write the set defining the model's years
fc = file(output.file,"w")
writeLines("sets",con=fc)
t = sage.years[1]
if (length(sage.years)>1)
  for (i in 2:length(sage.years))
    t = paste0(t,", ",sage.years[i])
writeLines(paste0("  t                     time periods            / ",t," /;"),
           con=fc)
writeLines("",con=fc)
close(fc)



# ------------------------------------------------------------------------------
# labor productivity
# ------------------------------------------------------------------------------

# get cbo long term outlook data
url = "https://www.cbo.gov/system/files/2020-09/"
file.name = "51119-2020-09-ltbo_0.xlsx"
download_file(url,file.name,dir=data.dir)

# load the sheet with the economic parameters
sheets = excel_sheets(file.path(data.dir,file.name))
econ.vars = read_excel(file.path(data.dir,file.name),
                       sheet=grep("economic vars",sheets,ignore.case=TRUE),
                       skip=6)

# the table is parameter by year but the parameter name can be in any of the
# first three columns so these need to be collapsed so the parameter name is
# always in the first column and the data starts in the second column
for (i in 1:nrow(econ.vars)) {
  var = c(econ.vars[i,1:3])[!is.na(econ.vars[i,1:3])]
  if (length(var)>0)
    econ.vars[i,1] = var
}

# remove any rows without data
econ.vars = econ.vars[!is.na(econ.vars[,1]),]
econ.vars = econ.vars[rowSums(!is.na(econ.vars[,-1]))>=1,]

# only keep the data for years in the model
econ.vars = econ.vars[,c(TRUE,names(econ.vars)[-1] >= min(sage.years) &
                              names(econ.vars)[-1] <= max(sage.years))]

# extract the labor productivity growth rate
row = grep("labor force productivity",t(econ.vars[,1]),ignore.case=TRUE)[1]
lp.cbo = data.frame(t=as.numeric(names(econ.vars)[-1]),
                    value=as.numeric(t(econ.vars[row,-1]))/100)

# cbo growth rates are those occuring during that year and we need our index to
# represent the situation by the end of the sage year so for calculating the
# index the simplest approach is to just assign the cbo growth rate to the
# prior year
lp.cbo$t = lp.cbo$t-1
lp.cbo = lp.cbo[lp.cbo$t>=sage.years[1],]

# add extra years needed for the forecast planning
extra_years = ((max(lp.cbo$t)+1):max(max(sage.years),end_year))
lp.cbo = rbind(lp.cbo,data.frame(t=extra_years,
                                value=lp.cbo$value[nrow(lp.cbo)]))

# implement a linear decline
start_ind = which(lp.cbo$t==start_year)
end_ind = which(lp.cbo$t==end_year)
slope = lp.cbo$value[start_ind]/(end_year-start_year)
lp.cbo$value[start_ind:end_ind] = lp.cbo$value[start_ind]-
                           slope*(lp.cbo$t[start_ind:end_ind]-
                                  lp.cbo$t[start_ind])
lp.cbo$value[lp.cbo$t>end_year] = 0



# ------------------------------------------------------------------------------
# historical sectoral labor productivity growth
# ------------------------------------------------------------------------------

# set the name of the integrated productivity measure to pull from the bls-bea
# integrated estiamtes (labor or multifactor)
prod_measure = "Integrated Labor Productivity"

# download the integrated productivity estiamtes
url = "https://www.bls.gov/mfp/"
file.name = "indprod8.xlsx"
download_file(url,file.name,dir=data.dir)

# load the productivity estiamtes
data = read_excel(file.path(data.dir,file.name),sheet=prod_measure,
                  skip=1)
data = data[!is.na(data[,2]),]
names(data)[1] = "description"

# load the naics concordance
naics = read_excel(file.path(data.dir,file.name),sheet="NAICS codes",skip=2)
naics = naics[!is.na(naics[,3]),c(1,3)]
names(naics) = c("description","naics")

# load the gross output estiamtes
output =  read_excel(file.path(data.dir,file.name),sheet="Gross Output",
                     skip=1)
names(output)[1] = "description"

# convert to growth rate
data[,-(1:2)] = data[,-(1:2)]/data[,-c(1,ncol(data))]-1
data = data[,-2]

# load the naics to sage sectoral mapping
naics_to_sage = read.csv(file.path(data.dir,"naics_to_sage.csv"),
                         stringsAsFactors=FALSE)

# create a mapping that has a row for each time one of the bls-bea industries
# shows up in a sage industry based on naics codes - note that this approach
# means that one of the bls-bea industries can influence the growth in more than
# one sage industry
mapping = NULL
for (row in 1:nrow(naics)) {
  codes = trimws(unlist(strsplit(as.character(naics[row,2]),",")))
  for (code in codes) {
    temp = unlist(strsplit(code,"-"))
    if (length(temp)>1) {
      codes = c(codes,as.character(temp[1]:temp[2]))
      codes = codes[codes!=code]
    }
  }
  codes = codes[codes!="NA"]
  for (code in codes) {
    sage = unique(naics_to_sage$sage[substr(naics_to_sage$naics,
                                            1,nchar(code))==code])
    if (length(sage)>=1)
      mapping = rbind(mapping,
                      data.frame(description=naics$description[row],
                                 sage=sage))
  }
}
mapping = unique(mapping)

# make sure that the gross output data has the same set of years as the
# productivity data
output = output[,names(output) %in% names(data)]

# for each sage sector use the gross output weighted average productivity growth
# rate across the average bls-bea sectoral growth rates
lp = NULL
for (sector in unique(mapping$sage)) {
  descriptions = mapping$description[mapping$sage==sector]
  rows = data$description %in% descriptions
  lp = rbind(lp,
             data.frame(sector=sector,
                        value=mean(colSums(data[rows,-1]*output[rows,-1])/
                                     colSums(output[rows,-1]))))
}

# store the historic labor productivity growth rates so they can be used in
# comparison plots for the documentation
write.csv(lp,file.path("documentation","hist_prod_growth.csv"),row.names=FALSE)



# ------------------------------------------------------------------------------
# scale sector labor productivity growth rates by cbo forecast
# ------------------------------------------------------------------------------

# load the benchmark labor demand for sage
ld0 = rgdx.param(file.path("data","default_aggregation.gdx"),"ld0")

# aggregate benchmark labor demand across regions
ld0 = aggregate(list(ld0=ld0$ld0),by=list(sector=ld0$s),sum)

# convert sector names to characters - sets are factors by default from rgdx
ld0$sector = as.character(ld0$sector)

# total economy wide benchmark labor demand
ld0_total = sum(ld0$ld0)

# function returning error on scaling parameter for sector specific labor
# productivity growth rates to match the aggregate cbo forecast - based on
# assuming relative sectoral output and labor inputs remain the same
objective = function(x,ld0)
  sum(ld0$ld0/(ld0$value*(1+x*ld0$lp)))-
    sum(ld0$ld0)/prod(1+lp.cbo$value[lp.cbo$t<=t])

# initalize a container for the calibrated sector specific productivity growth
lp.cal = expand.grid(lp.cbo$t,lp$sector)
names(lp.cal) = c("t","sector")
lp.cal$value = 1

# for each year calibrate the sector specific productivity growth to be
# consistent with the aggregate productivity growth forecast from cbo
for (t in lp.cbo$t[-1]) {

  # implement the convergence from the historic sector specific productivity
  # growth rate as a linear decline in the variance from converge_Start to
  # converge_end years
  current.lp = lp
  if (t<converge_start) {
    current.lp$value = lp$value
  } else if (t>=converge_start & t<=converge_end) {
    current.lp$value = lp$value-(lp$value-mean(lp$value))/
                                  (converge_end-converge_start)*
                                  (t-converge_start)
  } else {
    current.lp$value = mean(lp$value)
  }
  names(current.lp)[names(current.lp)=="value"] = "lp"

  # create a data frame with the benchmark labor demand, sector specific
  # productivity growth for the current year including any convergence, and
  # the previous cumulative productivity growth in the sectors - all used in
  # the routine to calibrate to the aggregate labor productivity for the current
  # year
  current.ld = merge(merge(ld0,current.lp),
                     lp.cal[lp.cal$t==t-1,c("sector","value")])

  # solve for the scaler for the sector specific productivity growth including
  # any convergence to calibrate to the aggregate productivity growth projection
  scaler = uniroot(objective,c(-10,100),current.ld)$root

  # save the cumulative growth for the current year
  lp.cal$value[lp.cal$t==t] = lp.cal$value[lp.cal$t==t-1]*
                              (1+scaler*current.lp$lp)

}

# only keep the relevant sage years
lp.cal = lp.cal[lp.cal$t %in% sage.years,]

write.gams.variable(output.file,"l_prod",lp.cal,append=TRUE,
                    comments="baseline - sector labor productivity index")



# ------------------------------------------------------------------------------
# labor force
# ------------------------------------------------------------------------------

# extract the labor force growth rate
row = grep("growth of the labor force",t(econ.vars[,1]),ignore.case=TRUE)[1]
lf.cbo = data.frame(t=as.numeric(names(econ.vars)[-1]),
                    value=as.numeric(t(econ.vars[row,-1]))/100)

# cbo growth rates are those occuring during that year and we need our index to
# represent the situation by the end of the sage year so for calculating the
# index the simplest approach is to just assign the cbo growth rate to the
# prior year
lf.cbo$t = lf.cbo$t-1
lf.cbo = lf.cbo[lf.cbo$t>=sage.years[1],]

# add extra years needed for the forecast planning
extra_years = ((max(lf.cbo$t)+1):max(max(sage.years),end_year))
lf.cbo = rbind(lf.cbo,data.frame(t=extra_years,
                                value=lf.cbo$value[nrow(lf.cbo)]))

# implement a linear decline
start_ind = which(lf.cbo$t==start_year)
end_ind = which(lf.cbo$t==end_year)
slope = lf.cbo$value[start_ind]/(end_year-start_year)
lf.cbo$value[start_ind:end_ind] = lf.cbo$value[start_ind]-
                            slope*(lf.cbo$t[start_ind:end_ind]-
                                   lf.cbo$t[start_ind])
lf.cbo$value[lf.cbo$t>end_year] = 0

# convert to an index relative to the benchmark year
lf.cbo$value = c(1, cumprod(1+lf.cbo$value[-nrow(lf.cbo)]))

# convert to an index relative to the benchmark year
lp.cbo$value = c(1, cumprod(1+lp.cbo$value[-nrow(lp.cbo)]))



# ------------------------------------------------------------------------------
# gdp - used to set other exogenous projections
# ------------------------------------------------------------------------------

# extract real gdp
row = grep("real gdp [(]billions",t(econ.vars[,1]),ignore.case=TRUE)[1]
gdp = data.frame(t=as.numeric(names(econ.vars)[-1]),
                 value=as.numeric(t(econ.vars[row,-1])))

# add extra years needed for the forecast planning
extra_years = ((max(gdp$t)+1):max(sage.years))

get.index.growth = function(x)
  x$value[x$t %in% extra_years]/x$value[x$t==gdp$t[nrow(gdp)]]

gdp = rbind(gdp,data.frame(t=extra_years,
                           value=gdp$value[nrow(gdp)]*get.index.growth(lp.cbo)*
                                   get.index.growth(lf.cbo)))

# extract the gdp price index
row = grep("Growth of the GDP price index",t(econ.vars[,1]),ignore.case=TRUE)[1]
gdp_price_index = data.frame(t=as.numeric(names(econ.vars)[-1]),
                             value=as.numeric(t(econ.vars[row,-1])))

# convert from growth rate to an index relative to benchmark year prices
gdp_price_index$value[-1] = 1/cumprod(1+gdp_price_index$value[-nrow(gdp_price_index)]/100)
gdp_price_index$value[1] = 1

# only keep the relevant sage years
lp.cbo = lp.cbo[lp.cbo$t %in% sage.years,]

write.gams.variable(output.file,"l_prod_agg",lp.cbo,append=TRUE,
                    comments="baseline - aggregate labor productivity index",
                    write.zeros=TRUE)

# only keep the relevant sage years
lf.cbo = lf.cbo[lf.cbo$t %in% sage.years,]

write.gams.variable(output.file,"l_force",lf.cbo,append=TRUE,
                   comments="baseline - labor force index",
                   write.zeros=TRUE)



# ------------------------------------------------------------------------------
# current account balance
# ------------------------------------------------------------------------------

# cbo forecast file info
cbo_url = "https://www.cbo.gov/system/files/2020-07/"
cbo_zip_file = "55022-2020-07-historicaleconomicdata.zip"
cbo_zip_dir = substr(cbo_zip_file,regexpr("-",cbo_zip_file,fixed=T)[1]+1,
                                  regexpr(".",cbo_zip_file,fixed=T)[1]-1)
cbo_file = file.path(cbo_zip_dir,"Annual_CY_July20.csv")

# load the cbo forecast, downloading file if not already present
download_file(cbo_url,cbo_zip_file,dir=data_dir,check.file=cbo_file)
cbo_data_hist = read.csv(file.path(data_dir,cbo_file),stringsAsFactors=FALSE)

# final year in the cbo forecast
cbo_end_year = max(cbo_data_hist$date)

# create a data.frame with just the current account balance
cab = cbind(cbo_data_hist$date, cbo_data_hist$current_account_balance)

# add extra years needed for the forecast planning
extra_years = (cbo_end_year+1):max(max(sage.years),decline_end_year)
cab = rbind(cab,cbind(extra_years,cab[nrow(cab),2]))

# start and end rows for the linear decline to zero
start_ind = which(cab[,1]==(cbo_end_year+1))
end_ind = which(cab[,1]==decline_end_year)

# implement a linear decline in the current account balance to gdp ratio
slope = cab[start_ind,2]/(decline_end_year-(cbo_end_year+1))
cab[start_ind:end_ind,2] = cab[start_ind,2]-
                             slope*(cab[start_ind:end_ind,1]-
                                    cab[start_ind,1])

# set the current account balance to zero after the linear decline
cab[cab[,1]>decline_end_year,2] = 0

# only keep the relevant sage years
cab = data.frame(t=sage.years,value=cab[cab[,1] %in% sage.years,2])

# set the values relative to the benchmark year
cab$value = cab$value/cab$value[1]

write.gams.variable(output.file,"curactbal",cab,append=TRUE,
                    comments="baseline - current account balance index",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# government decifict and debt
# ------------------------------------------------------------------------------

# cbo forecast file info
cbo_url = "https://www.cbo.gov/system/files/2020-09/"
cbo_file = "51118-2020-09-budgetprojections.xlsx"

# load the cbo forecast, downloading file if not already present
download_file(cbo_url,cbo_file,dir=data_dir)
cbo_data = as.data.frame(read_excel(file.path(data_dir,cbo_file),
                                    sheet="Table 1",skip=8),
                         stringsAsFactors=FALSE)

# reset na's as blank strings
cbo_data[is.na(cbo_data)] = ""

# combine the three columns of row names into just one
row_names = trimws(do.call(paste,c(cbo_data[,(1:4)],sep="")))
cbo_data = cbo_data[,-(1:3)]
cbo_data[,1] = row_names

# extract the cbo forecast of the deficit
deficit = data.frame(year=as.numeric(substr(names(cbo_data[,-1]),1,4)),
                     stringsAsFactors=FALSE)
deficit$value = as.numeric(c(cbo_data[which(cbo_data[,1]=="Deficit (-) or Surplus")[1],
                         2:ncol(cbo_data)]))
deficit = deficit[2:12,]

# cbo historic file info
cbo_url = "https://www.cbo.gov/system/files/2020-01/"
hist_file = "51134-2020-01-historicalbudgetdata.xlsx"

# load the cbo historic data, downloading file if not already present
download_file(cbo_url,hist_file,dir=data_dir)
hist_data = as.data.frame(read_excel(file.path(data_dir,hist_file),
                                    sheet="1. Rev, Outlays, Surplus, Debt",
                                    skip=7),
                         stringsAsFactors=FALSE)
hist_data[,1] = as.numeric(hist_data[,1])
temp = hist_data[1:(which(is.na(hist_data[,1]))[1]-1),
                 c(1,which(names(hist_data)=="Total"))]
names(temp) = c("year","value")

# combine the historical data with the cbo forecast
deficit = rbind(temp,deficit)

# convert to real
to_real = function(x) {
  for (t in gdp_price_index$t) {
    row = x$year==t
    x$value[row] = x$value[row]*gdp_price_index$value[gdp_price_index$t==t]
  }
  return(x)
}
deficit = to_real(deficit)

# state and local government deficits as a share of gdp in the benchmark year
state_deficit = get.bea.variable("T30300",benchmark_year,row=44)[1]/
                get.bea.variable("T10105",benchmark_year,row=1)[1]

# create a data.frame with just the state and local deficit assuming that it
# grows with gdp over the time frame of the cbo forecast
state_deficit = data.frame(year=cbo_data_hist$date,
                           value=cbo_data_hist$gdp*state_deficit)
state_deficit = state_deficit[state_deficit$year %in% deficit$year,]

# combine the federal, state, and local deficits
deficit$value = deficit$value+state_deficit$value

# add extra years needed for the forecast planning
deficit = rbind(deficit,data.frame(year=extra_years,
                                   value=deficit[nrow(deficit),2]))

# start and end rows for the linear decline to zero
start_ind = which(deficit[,1]==(cbo_end_year+1))
end_ind = which(deficit[,1]==decline_end_year)

# implement a linear decline in the current account balance to gdp ratio
slope = deficit[start_ind,2]/(decline_end_year-(cbo_end_year+1))
deficit[start_ind:end_ind,2] = deficit[start_ind,2]-
                             slope*(deficit[start_ind:end_ind,1]-
                                    deficit[start_ind,1])

# set the current account balance to zero after the linear decline
deficit[deficit[,1]>decline_end_year,2] = 0

# load the federal plus state and local debt in the benchmark year
debt0 = rgdx.scalar(file.path("data","default_aggregation.gdx"),"debt0")

# projection of debt based on the current debt level and the projected deficit
debt = deficit[deficit$year>=sage.years[1],]
debt$value = debt0-cumsum(c(0,debt$value[-nrow(debt)]))

# only keep the relevant sage years
deficit = data.frame(t=sage.years,value=deficit[deficit[,1] %in% sage.years,2])

# set the values relative to the benchmark year
deficit$value = deficit$value/deficit$value[1]

# write to the gams input file
write.gams.variable(output.file,"deficit",deficit,append=TRUE,
                    comments="baseline - deficit index",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# government interest payments
# ------------------------------------------------------------------------------

# net interest rate implicit in the benchmark sam
rate = -get.bea.variable("T30100",benchmark_year,row=43)[1]/1000/debt0

# get the projection of total interest payments
gint = debt
gint$value = debt$value*rate

# interest payments domestically
dshare = get.bea.variable("T30100",benchmark_year,row=28)[1]/
         (get.bea.variable("T30100",benchmark_year,row=29)[1]+
          get.bea.variable("T30100",benchmark_year,row=28)[1])

# get the projection of interest paid to rest of world holding their share of
# debt constant at the benchmark share
gint_row = gint
gint_row$value = (1-dshare)*gint$value
gint$value = gint$value-gint_row$value

# only keep the relevant sage years
gint = data.frame(t=sage.years,value=gint[gint[,1] %in% sage.years,2])
gint_row = data.frame(t=sage.years,value=gint_row[gint_row[,1] %in% sage.years,2])

# set the values relative to the benchmark year
gint$value = gint$value/gint$value[1]
gint_row$value = gint_row$value/gint_row$value[1]

# write to the gams input file
write.gams.variable(output.file,"gint",gint,append=TRUE,
                    comments="baseline - government interest payments domestically",
                    write.zeros=TRUE)
write.gams.variable(output.file,"gint_row",gint_row,append=TRUE,
                    comments="baseline - government interest payments to rest of world",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# government expenditure
# ------------------------------------------------------------------------------

# load the cbo forecast for government outlays
cbo_data = as.data.frame(read_excel(file.path(data_dir,cbo_file),
                                    sheet="Table 1",skip=8),
                         stringsAsFactors=FALSE)

# reset na's as blank strings
cbo_data[is.na(cbo_data)] = ""

# combine the three columns of row names into just one
row_names = trimws(do.call(paste,c(cbo_data[,(1:3)],sep="")))
cbo_data = cbo_data[,-(1:3)]
cbo_data[,1] = row_names

# extract the cbo forecast of discretionary outlays
gov = data.frame(year=as.numeric(substr(names(cbo_data[,-1]),1,4)),
                 stringsAsFactors=FALSE)
gov$value = as.numeric(cbo_data[grep("Discretionary",cbo_data[,1])[1],-1])
gov = gov[2:12,]

# load the cbo historic data
hist_data = as.data.frame(read_excel(file.path(data_dir,hist_file),
                                    sheet="5. Outlays",
                                    skip=7),
                         stringsAsFactors=FALSE)
hist_data[,1] = as.numeric(hist_data[,1])
temp = hist_data[1:(which(is.na(hist_data[,1]))[1]-1),
                 c(1,grep("Discretionary",names(hist_data)))]
names(temp) = c("year","value")

# combine the historical data with the cbo forecast
gov = rbind(temp,gov)

# convert to real
gov = to_real(gov)

# state and local government expenditures as a share of gdp in the benchmark
state_exp = get.bea.variable("T30300",benchmark_year,row=24)[1]/
            get.bea.variable("T10105",benchmark_year,row=1)[1]

# create a data.frame with just the state and local expenditures assuming that
# they grow with gdp over the time frame of the cbo forecast
state_exp = data.frame(year=cbo_data_hist$date,
                       value=cbo_data_hist$gdp*state_exp)
state_exp = state_exp[state_exp$year %in% gov$year,]

# combine the federal, state, and local expenditures
gov$value = gov$value+state_exp$value

# add extra years needed for the forecast planning
extra_years = (cbo_end_year+1):max(sage.years)
share = gov$value[nrow(gov)]/
        gdp$value[gdp$t==gov$year[nrow(gov)]]*
        gdp$value[gdp$t %in% extra_years]
gov = rbind(gov,data.frame(year=extra_years,value=share))

# only keep the relevant sage years
gov = data.frame(t=sage.years,value=gov$value[gov$year %in% sage.years])

# set the values relative to the benchmark year
gov$value = gov$value/gov$value[1]

# write to the gams input file
write.gams.variable(output.file,"gov_exp",gov,append=TRUE,
                    comments="baseline - government expenditures",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# government transfers to households
# ------------------------------------------------------------------------------

# load the cbo forecast for government outlays
cbo_data = as.data.frame(read_excel(file.path(data_dir,cbo_file),
                                    sheet="Table 3",skip=9),
                         stringsAsFactors=FALSE)

# reset na's as blank strings
cbo_data[is.na(cbo_data)] = ""

# combine the three columns of row names into just one
row_names = trimws(do.call(paste,c(cbo_data[,(1:4)],sep="")))
cbo_data = cbo_data[,-(1:4)]
cbo_data[,1] = row_names

# extract the cbo forecast of mandatory outlays without offsets - this is very
# close to the to what is include in the federl nipa transfers plus state grants
# in the benchmark
transfers = data.frame(year=as.numeric(substr(names(cbo_data[,-1]),1,4)),
                       stringsAsFactors=FALSE)
transfers$value = as.numeric(cbo_data[grep("Effects of Offsetting Receipts",
                                           cbo_data[,1]),-1])
transfers = transfers[2:12,]

# load the cbo historic data
hist_data = as.data.frame(read_excel(file.path(data_dir,hist_file),
                                    sheet="5. Outlays",
                                    skip=7),
                         stringsAsFactors=FALSE)
hist_data[,1] = as.numeric(hist_data[,1])
temp = hist_data[1:(which(is.na(hist_data[,1]))[1]-1),
                 c(1,grep("Programmatic Outlays",names(hist_data)))]
names(temp) = c("year","value")

# combine the historical data with the cbo forecast
transfers = rbind(temp,transfers)

# convert to real
transfers = to_real(transfers)

# add extra years needed for the forecast planning
extra_years = (cbo_end_year+1):max(sage.years)
share = transfers$value[nrow(transfers)]/
        gdp$value[gdp$t==transfers$year[nrow(transfers)]]*
        gdp$value[gdp$t %in% extra_years]
transfers = rbind(transfers,data.frame(year=extra_years,value=share))

# only keep the relevant sage years
transfers = data.frame(t=sage.years,
                       value=transfers$value[transfers$year %in% sage.years])

# set the values relative to the benchmark year
transfers$value = transfers$value/transfers$value[1]

# write to the gams input file
write.gams.variable(output.file,"transfers",transfers,append=TRUE,
                    comments="baseline - government transfer payments",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# net of other government adjustments
# ------------------------------------------------------------------------------

# other adjustments between government and households follow gdp
oth_adj = data.frame(t=sage.years,
                     value=gdp$value[gdp$t %in% sage.years])

# set the values relative to the benchmark year
oth_adj$value = oth_adj$value/oth_adj$value[1]

# write to the gams input file
write.gams.variable(output.file,"oth_adj",oth_adj,append=TRUE,
                    comments="baseline - net of other government adjustments",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# rest of world net income, taxes, and transfers
# ------------------------------------------------------------------------------

# based on gdp growth
inc_row = gdp[gdp$t %in% sage.years,]
tran_row = gdp[gdp$t %in% sage.years,]

# set the values relative to the benchmark year
inc_row$value = inc_row$value/inc_row$value[1]
tran_row$value = tran_row$value/tran_row$value[1]

# write to the gams input file
write.gams.variable(output.file,"inc_row",inc_row,append=TRUE,
                    comments="baseline - net income receipts from rest of world",
                    write.zeros=TRUE)

write.gams.variable(output.file,"tran_row",tran_row,append=TRUE,
                    comments="baseline - net tax and transfer receipts from rest of world",
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# average energy intensity growth rate
# ------------------------------------------------------------------------------

# baseline improvements in the energy intensity of production are based on the
# unit energy consumption in eia's aeo forecast. for each sage sector we compute
# the gorwth rate of consumption per unit of real output aggregated across the
# nems sectors within that sage sector over the aeo time horizon. the mapping of
# nems to sage sectors is located in build/data/nems_to_sage.csv

# initialize space for the value of shipements and energy consumption data
output = NULL
energy = NULL

ids = read.csv(file.path("build","data","aeo_api_ids_by_sector.csv"),
               stringsAsFactors=FALSE)

# for each nems sector retrieve the data using the eia api
for (i in 1:nrow(ids)) {

  # add the output data for the sector to the dataset
  id = paste0("AEO.",aeo.year,".",aeo.scenario,".",ids$y[i],".A")
  temp = get.eia(id)
  temp = temp[seq(nrow(temp),1),]
  data.versions = rbind(data.versions,get.eia.version(id))
  temp$sector = ids$nems[i]
  output = rbind(output,temp)

  # add the energy consumption data for the sector to the dataset
  for (j in 3:ncol(ids)) {
    if (is.na(ids[i,j]))
      next
    id = paste0("AEO.",aeo.year,".",aeo.scenario,".",ids[i,j],".A")
    temp = get.eia(id)
    temp = temp[seq(nrow(temp),1),]
    data.versions = rbind(data.versions,get.eia.version(id))
    temp$sector = ids$nems[i]
    temp$source = names(ids)[j]
    energy = rbind(energy,temp)
  }

}

# load the nems to sage mapping
map = read.csv(file.path(data.dir,"nems_to_sage.csv"),
               stringsAsFactors=FALSE,header=TRUE)

# get the list of sage sectors
sage.sectors = unique(map$sage)

# for each sage sector get the energy intensities
ene_growth = NULL
for (s in sage.sectors) {

  if (any(map$nems[map$sage==s]==""))
    next

  # get energy and output for the nems sectors in the sage sector
  sector.energy = energy[energy$sector %in% map$nems[map$sage==s],]
  sector.output = output[output$sector %in% map$nems[map$sage==s],]

  # aggregate across nems sectors by year
  sector.energy = aggregate(sector.energy$value,by=list(sector.energy$year),sum)
  sector.output = aggregate(sector.output$value,by=list(sector.output$year),sum)

  names(sector.energy) = c("year","value")
  names(sector.output) = c("year","value")

  # average energy intensity growth rate
  intensity = sector.energy
  intensity$value = sector.energy$value/sector.output$value
  intensity$value = intensity$value/intensity$value[intensity$year==benchmark_year]

  # add the extra years in the sage baseline assuming no further change
  extra_years = (max(intensity$year)+1):max(sage.years)
  intensity = rbind(intensity,data.frame(year=extra_years,
                    value=intensity$value[intensity$year==max(intensity$year)]))

  # add the sector
  intensity$s = s

  for (ss in c("gas","ele","col","ref")) {
    intensity$ss = ss
    ene_growth = rbind(ene_growth,intensity[,c("year","ss","s","value")])
  }

}

ene_growth = ene_growth[!is.nan(ene_growth$value),]

# energy intensity growth for the transportation sector is based on fuel
# efficiency improvements forecast for airline travel as reported in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_AIR_USE_NA_NA_QBTU.A")
temp = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".KEI_TRV_TRN_NA_AIR_NA_NA_BLNSEATMLS.A")
temp$value = temp$value/get.eia(id)$value
temp = temp[seq(nrow(temp),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# set the index relative to the benchmark year
temp$value = temp$value/temp$value[temp$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(temp$year)+1):max(sage.years)
temp = rbind(temp,data.frame(year=extra_years,
                             value=temp$value[temp$year==max(temp$year)]))

# store the projection
temp$ss = "ref"
temp$s = "trn"
ene_growth = rbind(ene_growth,temp[,c("year","ss","s","value")])

# energy intensity growth for the truck transportation sector is based on fuel
# efficiency improvements forecast for frieght truck travel as reported in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_FGHT_USE_NA_NA_QBTU.A")
temp = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".KEI_TRV_TRN_NA_FGHT_NA_NA_BLNVEHMLS.A")
temp$value = temp$value/(get.eia(id)$value)
temp = temp[seq(nrow(temp),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# set the index relative to the benchmark year
temp$value = temp$value/temp$value[temp$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(temp$year)+1):max(sage.years)
temp = rbind(temp,data.frame(year=extra_years,
                             value=temp$value[temp$year==max(temp$year)]))

# store the projection
temp$ss = "ref"
temp$s = "ttn"
ene_growth = rbind(ene_growth,temp[,c("year","ss","s","value")])

# only keep the relevant sage years
ene_growth = ene_growth[ene_growth$year %in% sage.years,]

write.gams.variable(output.file,"ene_growth",ene_growth,
                    comments="baseline - energy efficiency index",
                    write.zeros=TRUE,
                    append=TRUE)



# ------------------------------------------------------------------------------
# average growth rate of coal fired electricity generation share
# ------------------------------------------------------------------------------

# electricity output valued at initial year prices
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".GEN_NA_ELEP_CHP_TTG_NA_USA_BLNKWH.A")
ele = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# coal input valued at initial year prices
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_ENU_ELEP_NA_STC_NA_NA_QBTU.A")
#id = paste0("AEO.",aeo.year,".",aeo.scenario,
#            ".CNSM_NA_ELEP_NA_CL_NA_NA_MILLTON.A")
col = get.eia(id)
data.versions = rbind(data.versions,get.eia.version(id))

# coal fired electricity generation share
col_ele = col
col_ele$value = col$value/ele$value
col_ele = col_ele[seq(nrow(col_ele),1),]

# convert to an index
col_ele$value = col_ele$value/col_ele$value[col_ele$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(col_ele$year)+1):max(sage.years)
col_ele = rbind(col_ele,
                data.frame(year=extra_years,
                           value=col_ele$value[col_ele$year==max(col_ele$year)]))

# only keep the relevant sage years
col_ele = col_ele[col_ele$year %in% sage.years,]

write.gams.variable(output.file,"col_ele_growth",col_ele,
                    comments="baseline - coal share of electricity generation index",
                    write.zeros=TRUE,
                    append=TRUE)



# ------------------------------------------------------------------------------
# household energy intensity growth
# ------------------------------------------------------------------------------

# consumption forecast in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".ECI_NA_NA_NA_GDP_RCNSP_NA_BLNY09DLR.A")
c = get.eia(id)
c = c[seq(nrow(c),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# consumption electricity intensity growth is based on residential electricity
# consumption growth in aeo relative to consumption growth
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_RESD_NA_ELS_NA_USA_BLNKWH.A")
e = get.eia(id)
e = e[seq(nrow(e),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
c.share = c
c.share$value = e$value/c$value
c.share$value = c.share$value/c.share$value[c.share$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(c.share$year)+1):max(sage.years)
c.share = rbind(c.share,
                data.frame(year=extra_years,
                           value=c.share$value[c.share$year==max(c.share$year)]))

# store the projection
c.share$s = "ele"
cd_ene_growth = c.share

# consumption natural gas intensity growth is based on residential natural gas
# consumption growth in aeo relative to consumption growth
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_ENU_RESD_NA_NG_NA_NA_QBTU.A")
e = get.eia(id)
e = e[seq(nrow(e),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# set the growth rate
# the eia api returns the last year on top so temp$value[1] is the last year
c.share = c
c.share$value = e$value/c$value
c.share$value = c.share$value/c.share$value[c.share$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(c.share$year)+1):max(sage.years)
c.share = rbind(c.share,
                data.frame(year=extra_years,
                           value=c.share$value[c.share$year==max(c.share$year)]))

# store the projection
c.share$s = "gas"
cd_ene_growth = rbind(cd_ene_growth,c.share)

# consumption refined oil intensity growth is based on light duty vehicle energy
# intensity growth in aeo
id = paste0("AEO.",aeo.year,".",aeo.scenario,
            ".CNSM_NA_TRN_LDV_USE_NA_NA_QBTU.A")
e = get.eia(id)
e = e[seq(nrow(e),1),]
data.versions = rbind(data.versions,get.eia.version(id))

# implicitly assumes that households share of overall light duty vehicle miles
# traveled will remain constant
c.share = c
c.share$value = e$value/c$value
c.share$value = c.share$value/c.share$value[c.share$year==benchmark_year]

# add the extra years in the sage baseline assuming no further change
extra_years = (max(c.share$year)+1):max(sage.years)
c.share = rbind(c.share,
                data.frame(year=extra_years,
                           value=c.share$value[c.share$year==max(c.share$year)]))

# store the projection
c.share$s = "ref"
cd_ene_growth = rbind(cd_ene_growth,c.share)

# zero out any change in coal
c.share$s = "col"
c.share$value = 1
cd_ene_growth = rbind(cd_ene_growth,c.share)

# only keep the relevant sage years
cd_ene_growth = cd_ene_growth[cd_ene_growth$year %in% sage.years,
                              c("year","s","value")]

write.gams.variable(output.file,"cd_ene_growth",cd_ene_growth,
                    comments="baseline - household energy intensity growth",
                    write.zeros=TRUE,
                    append=TRUE)



# ------------------------------------------------------------------------------
# save the r data image for versioning and generate report
# ------------------------------------------------------------------------------

# save data versioning eia parameter
data.versions = data.frame(data.versions)
colnames(data.versions)[1] = "agency"
colnames(data.versions)[2] = "call_id"
colnames(data.versions)[3] = "version"
write.table(data.versions,"build/data/satellite_data_versions.csv",sep = ",",
            col.names=!file.exists("build/data/satellite_data_versions.csv"),
            append=TRUE)

# save data image for posterity
save.image(file=paste0("build/data/build_baseline_image.RData"))
