# build_parameters.R
#
#



# ------------------------------------------------------------------------------
# set general parameters
# ------------------------------------------------------------------------------

# relative path to where the data files are stored
data.dir = file.path("build","data")

# name of file to write the elasticities
output.file = file.path("data","parameters.gms")



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

library(httr)
library(RJSONIO)

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))



# ------------------------------------------------------------------------------
# large open economy elasticities
# ------------------------------------------------------------------------------

## load elasticity file
loe = read.csv(file.path(data.dir,'gtap_loe_elasticities.csv'),
               header=TRUE,stringsAsFactors=FALSE)

## make sure all sector names are lowercase
names(loe)[1] <- "s"
loe$s = tolower(loe$s)

## break up elasticities into export parameter and import parameter
eps_ex = loe[,c("s","exports")]
eps_im = loe[,c("s","imports")]

## note that GTAP does not distinguish between chemicals and plastics and
## rubber manufacturing. append a row assuming equivalent values.
eps_ex = rbind(eps_ex, data.frame(s="prm",exports=eps_ex[eps_ex$s=="chm","exports"]))
eps_im = rbind(eps_im, data.frame(s="prm",imports=eps_im[eps_im$s=="chm","imports"]))



# ------------------------------------------------------------------------------
# armington elasticities
# ------------------------------------------------------------------------------

# load the gtap elasticities
file.name = "hertel et al - GTAP v7 - behavioral parameters - table 14-2.csv"
gtap = read.csv(file.path(data.dir,file.name),
                stringsAsFactors=FALSE,header=TRUE,skip=19)

# load the gtap to sage mapping
gtap.map = read.csv(file.path(data.dir,"gtap_to_sage.csv"),
                    stringsAsFactors=FALSE,header=TRUE,skip=1)

# get the list of sage sectors
sage.sectors = unique(gtap.map$sage)

# for each sage sector get the armington elasticities
se_nf = NULL
se_dn = NULL
for (s in sage.sectors) {

  # gtap sectors within the current sage sector
  ind = gtap$GTAP.Sector %in% gtap.map$gtap[gtap.map$sage==s]

  # compute the value weighted national-foreign armington elasticity
  # weighting is by import value at world prices as reported in GTAP v9
  se_nf_value = sum(gtap$sigma_D[ind]*gtap$viws[ind])/sum(gtap$viws[ind])

  # crude oil and natural gas elasticities from hertel et al. (2007) are less
  # appropriate for the U.S. and therefore, we follow caron and rausch (2013)
  # in there updating of the hertel et al. (2007) elasticities for these sectors
  if (s=="cru")
    se_nf_value = 7.3
  if (s=="gas")
    se_nf_value = 2.8

  se_nf = rbind(se_nf,cbind(s,se_nf_value))

  # coughlin and novy (2013) ratio of intra-national to international border
  # effects
  delta = 1.868

  # caron and rausch (2013) relationship between domestic-national and national-
  # foreign armington elasticities
  se_dn = rbind(se_dn,cbind(s,.5-delta*(.5-se_nf_value)))

}



# ------------------------------------------------------------------------------
# transformation elasticity of output
# ------------------------------------------------------------------------------

# domestic vs. export transformation elasticity of output
# taken from us-rep
te_dx = 2



# ------------------------------------------------------------------------------
# transformation elasticity for sector differentiated extant capital
# ------------------------------------------------------------------------------

# transformation elasticity for sector differentiated extant capital
te_k_ex = 1.5



# ------------------------------------------------------------------------------
# substitution elasticity for aggregate investment good
# ------------------------------------------------------------------------------

# elasticity governing composition of aggregate regional investment
# reference value is set to be small to approximate leontief assumption
se_inv = 0.05



# ------------------------------------------------------------------------------
# klem substitution elasticities
# ------------------------------------------------------------------------------

# load the koesler and schymura (2015) elasticities
klem = read.csv(file.path(data.dir,
                          "koesler and schymura - 2015 - klem elasticities.csv"),
                stringsAsFactors=FALSE,header=TRUE,skip=8)

# for sectors without kl elasticities (either due to inf answers or
# convergencece failure na) and without other sectors being mapped to the sage
# sector use the young (2013) gmm non-normalized values instead
klem$se_kl[klem$nace=="23"] = 0.732      # petroleum refining
klem$se_kl[klem$nace=="E"]  = 1.002      # water and electric utilities

# petroleum refining has an inf value for the kle elasticity, instead use the
# the total industries value
klem$se_kle[klem$nace=="23"] = klem$se_kle[klem$nace=="TOT"]

# load the nace code to sage mapping
nace.map = read.csv(file.path(data.dir,"nace_to_sage.csv"),
                    stringsAsFactors=FALSE,header=TRUE)

# for each sage sector get the value added elasticity
se_kl = NULL
se_kle = NULL
se_klem = NULL
for (s in sage.sectors) {

  # nace sectors within the current sage sector
  ind = klem[,1] %in% nace.map$nace[nace.map$sage==s]

  # compute the value weighted value added elasticity
  # weighting is by output in the last year of the koesler and schymura sample
  # only include non-NA and non-Inf values
  ind_kl = ind & !(is.na(klem$se_kl) | klem$se_kl=="Inf")
  y = sum(as.double(klem$output[ind_kl]))
  kl_value = sum(as.double(klem$se_kl[ind_kl])*as.double(klem$output[ind_kl]))/y

  # compute the remaining klem elasticities
  # weighting is by output in the last year of the koesler and schymura sample
  y = sum(as.double(klem$output[ind]))
  kle_value = sum(as.double(klem$se_kle[ind])*as.double(klem$output[ind]))/y
  klem_value = sum(as.double(klem$se_klem[ind])*as.double(klem$output[ind]))/y

  # write the klem elasticities to the output file
  se_kl = rbind(se_kl,cbind(s,kl_value))
  se_kle = rbind(se_kle,cbind(s,kle_value))
  se_klem = rbind(se_klem,cbind(s,klem_value))

}



# ------------------------------------------------------------------------------
# interfuel energy substitution elasticities
# ------------------------------------------------------------------------------

# mapping between energy commodities and seds energy commodity codes
fuel.map = data.frame(fuel=c("col","ref","gas","ele"),
                      seds=c("CL","PA","NG","ES"),
                      stringsAsFactors=FALSE)

# mapping between aggregate sector names and seds codes
sector.map = data.frame(sector=c("industrial","commercial","electricity",
                                 "transportation","residential"),
                        seds=c("I","C","E","A","R"),
                        stringsAsFactors=FALSE)

# create a container for the consumption data
en.consumption = expand.grid(sector.map$sector,fuel.map$fuel,
                             stringsAsFactors=FALSE)
names(en.consumption) = c("sector","fuel")

# get the seds national level consumption in btus of each energy commodity by
# aggregate sector in the model's benchmark year
en.consumption$value = 0
for (i in 1:nrow(en.consumption)) {
  if (en.consumption$sector[i]=="electricity" & en.consumption$fuel[i]=="ele")
    next
  if (en.consumption$sector[i]=="electricity")
    c.var = "IB"
  else
    c.var = "CB"
  en.consumption$value[i] =
    get.eia(paste0("SEDS.",
                   fuel.map$seds[fuel.map$fuel==en.consumption$fuel[i]],
                   sector.map$seds[sector.map$sector==en.consumption$sector[i]],
                   c.var,".US.A"),year=sage.years[1])$value
}

# get the seds national level price of each energy commodity by aggregate sector
# in the model's benchmark year
en.consumption$price = 0
for (i in 1:nrow(en.consumption)) {
  if (en.consumption$sector[i]=="electricity" & en.consumption$fuel[i]=="ele")
    next
  if (en.consumption$sector[i]=="electricity")
    p.var = "ID"
  else
    p.var = "CD"
  en.consumption$price[i] =
    get.eia(paste0("SEDS.",
                   fuel.map$seds[fuel.map$fuel==en.consumption$fuel[i]],
                   sector.map$seds[sector.map$sector==en.consumption$sector[i]],
                   p.var,".US.A"),year=sage.years[1])$value
}

# consumption cost by aggregate sector and fuel
en.consumption$cost = en.consumption$value*en.consumption$price

# create a container for the aggregate sector elasticities between primary
# energy sources
se_en_agg = data.frame(sector=unique(sector.map$sector),stringsAsFactors=FALSE)

# serletis et al. (2010) allen substitution elasticities between refined
# petroleum and natural gas for the industril sector from table 7
se_en_agg$value[se_en_agg$sector=="industrial"] = 0.331

# serletis et al. (2010) results for the residential and commerical sector
# suggest very little substitution possibility between refined petorleum and
# natural gas so the substitution elasticities are set commensurately
se_en_agg$value[se_en_agg$sector=="residential"] = 0.1
se_en_agg$value[se_en_agg$sector=="commercial"] = 0.1

# serletis et al. (2010) allen substitution elasticities between coal and
# natural gas for the electricity sector from table 10
se_en_agg$value[se_en_agg$sector=="electricity"] = 0.227

# transportation based on serletis et al (2010b) international study
se_en_agg$value[se_en_agg$sector=="transportation"] = 0.25

# create a container for the aggregate sector elasticities between primary
# energy and electricity
se_ene_agg = data.frame(sector=unique(sector.map$sector),stringsAsFactors=FALSE)

# serletis et al. (2010) allen substitution elasticities between refined
# petroleum and electricity and natural gas and electricity from tables 7-9
sigma_ref_ele = c(0.24,1.532,1.280)
names(sigma_ref_ele) = c("industrial","residential","commercial")
sigma_gas_ele = c(1.693,0.675,0.497)
names(sigma_gas_ele) = c("industrial","residential","commercial")

# variable to use when weighting the elasticities
weight.variable = "cost"

# for the industrial, residential, and commercial sectors set the elasticity to
# a weighted average of the gas to electricity and refined petroleum to
# electricity elasticities
for (s in c("industrial","residential","commercial")) {

  ind = en.consumption$sector==s

  se_ene_agg$value[se_ene_agg$sector==s] =
    (en.consumption[ind & en.consumption$fuel=="ref",weight.variable]*
       sigma_ref_ele[s]+
     en.consumption[ind & en.consumption$fuel=="gas",weight.variable]*
       sigma_gas_ele[s])/
    (en.consumption[ind & en.consumption$fuel=="ref",weight.variable]+
     en.consumption[ind & en.consumption$fuel=="gas",weight.variable])

}

# virtually eliminate substitution possibilities between electricity and primary
# energy inputs in the electricity sector
se_ene_agg$value[se_ene_agg$sector=="electricity"] = 0.01

# transportation based on serletis et al (2010b) international study
se_ene_agg$value[se_ene_agg$sector=="transportation"] = 0.25

# commercial elasticities apply to the service sectors
com = c("hlt","srv")

# transportation sectors
trn = c("trn","ttn")

# for each sage sector get the energy elasticities
se_en = NULL
se_ene = NULL
for (s in sage.sectors) {

  # use the commercial elasticity index for designated commercial sectors
  if (s %in% com)
    sector = "commercial"
  else if (s == "ele")
    sector = "electricity"
  else if (s %in% trn)
    sector = "transportation"
  else
    sector = "industrial"

  # save sector specific elasticities
  se_en = rbind(se_en,cbind(s,se_en_agg$value[se_en_agg$sector==sector]))
  se_ene = rbind(se_ene,cbind(s,se_ene_agg$value[se_ene_agg$sector==sector]))

}

# save the energy elasticities for households
se_cen = se_en_agg$value[se_en_agg$sector=="residential"]
se_cene = se_ene_agg$value[se_ene_agg$sector=="residential"]



# ------------------------------------------------------------------------------
# labor supply elasticities
# ------------------------------------------------------------------------------

# labor supple elasticity - substitution effect
# based on most recent cbo literature review midpoints
lse_sub = 0.20

# labor supple elasticity - income effect
# based on most recent cbo literature review midpoints
lse_inc = -0.05



# ------------------------------------------------------------------------------
# household consumption elasticities
# ------------------------------------------------------------------------------



# substitution elasticity across non-energy and energy goods
# from usrep
se_c = 0.25;

# substituion elasticity across non-energy goods
# from usrep
se_cm = 0.25;

# inverse elasticity of substitution of consumption over time
eta = 1.66;



# ------------------------------------------------------------------------------
# fixed factor shares in resource and agriculture sectors
# ------------------------------------------------------------------------------

# fixed factor resource share of capital for resource extraction and
# agrictultural and forestry sectors
res_share = cbind(c("col","cru","gas","min","agf"),c(0.40,.25,0.25,0.40,0.40))



# ------------------------------------------------------------------------------
# resource extraction and agricutlure sector supply elasticities
# ------------------------------------------------------------------------------

# long-run supply elasticity for resoursce extraction and agrictultural and
# forestry sectors
se_res_lr = cbind(c("col","cru","gas","min","agf"),c(2.4,0.15,0.5,5,0.4))



# ------------------------------------------------------------------------------
# resource supply elasticities for sectors with a fixed factor
# ------------------------------------------------------------------------------

# resource supply elasticity with respect to commodity price
re = cbind(c("col","cru","gas","min","agf"),.01)



# ------------------------------------------------------------------------------
# macroeconomic variables
# ------------------------------------------------------------------------------

# rate - depreciation rate
delta = 0.05

# rate - benchmark interest rate
rbar = 0.045



# ------------------------------------------------------------------------------
# write parameters to the input file
# ------------------------------------------------------------------------------

# open a connection to the output file
fc = file(output.file,"w")

# large open economy elasticities
comments = "large open economy export demand elasticities"
write.gams.variable(fc,"eps_ex",eps_ex,comments=comments,write.zeros=TRUE)
comments = "large open economy import supply elasticities"
write.gams.variable(fc,"eps_im",eps_im,comments=comments,write.zeros=TRUE)

# armington elasticities
comments = "substitution elasticity - national-foreign armington elasticity"
write.gams.variable(fc,"se_nf",se_nf,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - domestic-national armington elasticity"
write.gams.variable(fc,"se_dn",se_dn,comments=comments,write.zeros=TRUE)

# transformation elasticity - domestic and exported production
comments = "transformation elasticity - domestic and exported production"
write.gams.variable(fc,"te_dx(s)",te_dx,comments=comments,write.zeros=TRUE)

# transformation elasticity for sector differentiated extant capital
comments = "transformation elasticity for sector differentiated extant capital"
write.gams.variable(fc,"te_k_ex",te_k_ex,comments=comments,write.zeros=TRUE)

# investment elasticity
comments = "substitution elasticity - investment bundle"
write.gams.variable(fc,"se_inv",se_inv,comments=comments,write.zeros=TRUE)

# klem substitution elasticities
comments = "substitution elasticity - value added"
write.gams.variable(fc,"se_kl",se_kl,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - value-added-energy bundle"
write.gams.variable(fc,"se_kle",se_kle,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - materials and value-added-energy bundle"
write.gams.variable(fc,"se_klem",se_klem,comments=comments,write.zeros=TRUE)

# energy substitution elasticities
comments = "substitution elasticity - primary energy bundle"
write.gams.variable(fc,"se_en",se_en,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - electricity and primary energy bundle"
write.gams.variable(fc,"se_ene",se_ene,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption primary energy bundle"
write.gams.variable(fc,"se_cen",se_cen,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption electricity and energy bundle"
write.gams.variable(fc,"se_cene",se_cene,comments=comments,write.zeros=TRUE)

# labor supply elasticities
comments = "labor supply elasticity - substitution"
write.gams.variable(fc,"lse_sub",lse_sub,comments=comments,write.zeros=TRUE)
comments = "labor supply elasticity - income"
write.gams.variable(fc,"lse_inc",lse_inc,comments=comments,write.zeros=TRUE)

# household consumption elasticities
comments = "substitution elasticity - consumption bundle"
write.gams.variable(fc,"se_c",se_c,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - consumption materials bundle"
write.gams.variable(fc,"se_cm",se_cm,comments=comments,write.zeros=TRUE)
comments = "substitution elasticity - inverse of consumption over time"
write.gams.variable(fc,"eta(r,h)",eta,comments=comments,write.zeros=TRUE)

# fixed factor shares in resource and agriculture sectors
comments = "fixed factor resource share of capital"
write.gams.variable(fc,"res_share",res_share,comments=comments,
                    write.zeros=TRUE)

# resource extraction and agriculture and forestry supply elasticities
comments = "long-run supply elasticity for sectors with fixed factor"
write.gams.variable(fc,"se_res_lr",se_res_lr,comments=comments,
                    write.zeros=TRUE)

# resource supply elasticities
comments = "resource supply elasticity with respect to commodity price"
write.gams.variable(fc,"re",re,comments=comments,write.zeros=TRUE)

# macroeconomic variables
comments = "rate - depreciation rate"
write.gams.variable(fc,"delta",delta,comments=comments,write.zeros=TRUE)
comments = "rate - benchmark interest rate"
write.gams.variable(fc,"rbar",rbar,comments=comments,write.zeros=TRUE)

# close the connection to the output file
close(fc)
