$stitle routine for calibrating subsistency demands for LES consumer demand

* this program calibrates non-leisure consumption to match income elasticities
* in the les consumer demand system.



* ------------------------------------------------------------------------------
* read in consumer demand data
* ------------------------------------------------------------------------------

* allow for end of line comments after a "!"
$eolcom !

* set the path separator based on the operating system
$set sep %system.dirsep%

* spatial aggregation
$if not set spatial $set spatial default

* set default benchmark file
$if not set benchmark_file $set benchmark_file "data%sep%%spatial%_aggregation.gdx"

* set default output file
$if not set output_file $set output_file "data%sep%%spatial%_aggregation.gdx"

* set default elasticity file
$if not set elasticity_file $set elasticity_file "build%sep%data%sep%%spatial%_aggregation_income_elasticities.csv"

* basic model sets
sets
  s                     sectors
  r                     regions
  h                     households
  trd                   trade markets
  f                     primary factors
  t                     time periods;

* set aliases
alias
  (s,ss,sss), (h,hh);

$gdxin %benchmark_file%
$load r s h trd f t

* benchmark data
parameters
  y0(r,s)               benchmark - output
  ld0(r,s)              benchmark - labor demand
  kd0(r,s)              benchmark - capital demand
  id0(r,s,ss)           benchmark - intermediate demand
  d0(r,s)               benchmark - domestic supply
  m0(r,s,trd)           benchmark - imports
  x0(r,s,trd)           benchmark - exports
  c0(r,h)               benchmark - consumption bundle
  cd0(r,s,h)            benchmark - consumer demand
  i0(r,s)               benchmark - investment demand
  invh0(r,h)            benchmark - household investment
  n0(r,h)               benchmark - households
  g0(r,s)               benchmark - government demand
  le0(r,h)              benchmark - household labor supply
  re0(r,h)              benchmark - capital rental income
  curactbal0            benchmark - current account balance
  inc_row0              benchmark - net income receipts from rest of world
  tran_row0             benchmark - net tax and transfer receipts from rest of world
  deficit0              benchmark - government deficit
  gint0                 benchmark - government interest payments domestically
  gint_row0             benchmark - government interest payments to rest of world
  debt0                 benchmark - government debt
  tran_fed0(r,h)        becnhmark - federal transfer payments to households
  tran_state0(r,h)      becnhmark - state transfer payments to households
  incadj0(r,h)          benchmark - government to household lump sum transfer
  ty0(r,s)              tax rate - production
  tp0(r)                tax rate - payroll tax
  tl0(r,h)              tax rate - labor income tax
  tfica0(r,h)           tax rate - fica tax
  tk0(r)                tax rate - capital income tax
  tc0(r)                tax rate - consumption
  tl_avg0(r,h)          tax rate - average labor income tax;

* Load benchmark data
$load id0 ld0 kd0 cd0 x0 m0 i0 g0 re0 le0 invh0 n0 ty0 tp0 tl0 tfica0 tc0 tk0
$load tl_avg0 tran_fed0 tran_state0 curactbal0 inc_row0 tran_row0 deficit0 gint0
$load gint_row0 debt0



* ------------------------------------------------------------------------------
* characterize income elasticities
* ------------------------------------------------------------------------------

set
  cex                   categories in the consumer expenditure survey data /
     alcbev             "alcoholic beverages",
     food               "food",
     tobacc             "tobacco",
     appar              "apparel",
     persca             "personal care",
     read               "reading/books",
     educa              "education",
     medical            "medical"
     entert             "entertainment",
     elec               "electricity",
     natural_gas        "natural gas",
     fuel_oil           "other heating fuels",
     telephone          "telephone",
     water              "water utilities",
     shelter            "rent for housing",
     oper_furnish       "house furnishings and equipment",
     vacation_home_q    "vacation home imputed rent",
     tran_fuel          "fuel for transportation",
     veh_mainten        "vehicle maintenance",
     veh_finance        "vehicle financing",
     veh_serv2          "vehicle services for owned or rented vehicles" /;

parameters
  regres                regression results from income elasticity estimation (based on aguiar et al. 2015),
  statsig               check for statistical significance,
  eta_(cex)             income elasticities for cex categories,
  pceshr(cex,s)         pce shares to link elasticities to sage demand categries,
  eta0(s)               reconciled income elasticities of demand matched to sage sectors;

* read in cex based income elasticities (estimated at the national level)
$call 'csv2gdx build%sep%data%sep%national_income_elasticities_CEX_2013_2017.csv output=build%sep%data%sep%national_income_elasticities_CEX_2013_2017.gdx id=regres index=1 useHeader=y value=(2,3,4)';
$gdxin 'build%sep%data%sep%national_income_elasticities_CEX_2013_2017.gdx'
$load regres

* report statistical significance of elasticity estimates and populate parameter
statsig(cex) = regres(cex,'elast') / regres(cex,'se');
eta_(cex) = regres(cex,'elast');

* read in constructed bridge file between cex and sage sectors (based on pce)
$call 'csv2gdx build%sep%data%sep%cex_sage_pcemap.csv output=build%sep%data%sep%cex_sage_pcemap.gdx id=pceshr index=(1,2) useHeader=y value=3';
$gdxin 'build%sep%data%sep%cex_sage_pcemap.gdx'
$load pceshr

* convert bridge file to shares (originally represented as percentages) and
* verify that shares sum to 1
parameters
  chkpce(s)             check pce shares
  chkmap(cex,s)         check on pce mapping;

pceshr(cex,s) = pceshr(cex,s) / 100;
chkpce(s) = sum(cex, pceshr(cex,s));
display chkpce;

* aggregate the elasticity estiamtes to sage sectors as weighted averages using
* expenditures in the pce
eta0(s) = sum(cex$pceshr(cex,s), pceshr(cex,s)*eta_(cex));

* set income elasticity to 1 for sectors that don't have a value
eta0(s)$(not eta0(s)) = 1;



* ------------------------------------------------------------------------------
* alter demand parameters to accomodate different demand systems
* ------------------------------------------------------------------------------

* linear expenditure system: construct QCP and solve for subsistence demands and
* calibrated shares given income elasticities and value shares.

parameters
  theta0(r,s,h)         reference budget shares
  theta_les(r,s,h)      value shares reflecting subsistence demand
  scd0(r,s,h)           subsistend demand
  national              switch for national demand /1/
  natscd0_              national subsistence demands (intermediary)
  natscd0(s)            national subsistence demands;

* if solving for subnational subsistence demands, read in national totals to
* constrain the calibration to match national totals
$if %spatial%=="national" national=0;
$if %spatial%=="national" natscd0(s)=1;
$if not %spatial%=="national" $gdxin "build%sep%data%sep%national_subsistence.gdx"
$if not %spatial%=="national" $load natscd0_=scd0
$if not %spatial%=="national" natscd0(s)$(not sameas(s,'col')) = natscd0_("us",s,"ra");

* define theta0 as the households share of commodity consumption going to good s
theta0(r,s,h) = cd0(r,s,h)/sum(ss, cd0(r,ss,h));

variable
  OBJ                   objective value;

nonnegative
variables
  ETA(r,s,h)            calibrated income elasticites
  SDM(r,s,h)            subsistence demand
  THETA(r,s,h)          demand exponent;

equations
  elast(r,s,h)          elasticity equation
  share(r,s,h)          share equations
  sumc(r,h)             summing constraint
  natcon(s)             aggregation constraint matching national totals
  objdef                least squares objective function;

* least squares objective function
objdef..
  OBJ =e= sum((r,s,h), sqr(ETA(r,s,h)/eta0(s) - 1));

* elasticity equation
elast(r,s,h)..
  ETA(r,s,h)*cd0(r,s,h) =e= THETA(r,s,h)*sum(ss, cd0(r,ss,h));

* share equations
share(r,s,h)..
  cd0(r,s,h)
    =e= SDM(r,s,h)+THETA(r,s,h)*(sum(ss, cd0(r,ss,h)) - sum(ss, SDM(r,ss,h)));

* summing constraint
sumc(r,h)..
  sum(s, THETA(r,s,h)) =e= 1;

* aggregation constraint matching national totals
natcon(s)$national..
  sum((r,h), SDM(r,s,h)) =e= natscd0(s);

* set upper bound on subsistence demand
SDM.UP(r,s,h) = cd0(r,s,h)/1.1;
SDM.L(r,s,h) = natscd0(s) / (card(r)*card(h));

model les / elast, share, sumc, objdef, natcon /;
solve les using QCP minimizing OBJ;

theta_les(r,s,h) = THETA.L(r,s,h);
scd0(r,s,h) = SDM.L(r,s,h);

parameters
  report                report subsistence vs. all consumption spending
  chketa                report calibrated vs. estimated income elasticities
  sumstat               summary statistics across households for each sector;

report(r,s,h,'subsistence') = scd0(r,s,h);
report(r,s,h,'consumption') = cd0(r,s,h);
report(r,s,h,'share')$cd0(r,s,h) = scd0(r,s,h) / cd0(r,s,h);
report(r,s,h,'inc_elast')$cd0(r,s,h) = ETA.L(r,s,h);
report(r,s,h,'own_price')$cd0(r,s,h) = - theta_les(r,s,h) *
    (1 - (sum(ss, scd0(r,ss,h)) - scd0(r,s,h))/sum(ss, cd0(r,ss,h))) /
    (cd0(r,s,h)/sum(ss, cd0(r,ss,h)));

chketa(r,s,h,'data') = eta0(s);
chketa(r,s,h,'cal') = ETA.L(r,s,h);
chketa(r,s,h,'pctdiff') = 100 * ETA.L(r,s,h) / eta0(s);

sumstat(s,'min') = smin((r,h), report(r,s,h,'share'));
sumstat(s,'max') = smax((r,h), report(r,s,h,'share'));
sumstat(s,'mean')$sum((r,h), report(r,s,h,'share')) =
    (1/sum((r,h)$report(r,s,h,'share'), 1)) * sum((r,h), report(r,s,h,'share'));
display report, chketa, sumstat;


* ------------------------------------------------------------------------------
* save benchmark dataset and estimated elasticities
* ------------------------------------------------------------------------------

* save dataset
execute_unload "%output_file%", f,s,r,h,t,id0,ld0,kd0,cd0,scd0,x0,m0,i0,g0,re0,
                                le0,invh0,n0,ty0,tp0,tl0,tfica0,tc0,tk0,trd,
                                tl_avg0,tran_fed0,tran_state0,gint0,gint_row0,
                                inc_row0,tran_row0,curactbal0,deficit0,debt0;

* for national data, report totals
$if %spatial%=="national" execute_unload "build%sep%data%sep%national_subsistence.gdx" scd0;

* define set to report results
set
  mrun /"data","cal","pctdiff"/;

* output csv results file
file les_results /"%elasticity_file%"/;
les_results.pc = 5;
les_results.nd = 6;
put les_results;

put "region", "sector", "household", "type", "value"/;
loop((r,s,h,mrun)$chketa(r,s,h,mrun), put r.tl, s.tl, h.tl, mrun.tl, chketa(r,s,h,mrun) / );
putclose;
