# get_tax_data: defines the tax rates for the model
#
# taxes defined:
#  tl                   tax rate - labor income tax
#  tfica                tax rate - fica tax
#  tk                   tax rate - capital income tax
#  tp                   tax rate - payroll tax
#  tc                   tax rate - consumption tax
#
# personal income taxes:
#  uses data from the cps to drive nber's taxsim model to obtain earnings
#  weighted average effective marginal labor income tax rates by state and
#  implan household groupings. personal income taxes on capital income are
#  aggregated to national level effective marginal tax rates.
#
# corporate income tax:
#  hard coded from cbo study on effective tax rate.
#
# payroll tax:
#  the payroll tax is set to zero in the default version of the model. both the
#  employee and employer parts of the fica tax are defined using tfica to
#  capture the cap on social security taxes, but this tax rate is kept in the
#  model in case there is a future need to break out medicare taxes from fica.
#
# consumption tax:
#  the consumption tax rate is set to zero in the default version of the model.
#  sales and excise taxes are included in the implan defined production tax
#  rates.
#
# notes on using the cps with nber's taxsim:
#  1. requires anthony damico's lodown package to easily grab the cps data from
#     census and import into r
#
#  2. uses the cps imputation of non-salt deductions for federal taxes when
#     possible
#
#  3. it is not clear if child cares expenses are included in the cps imputed
#     deductions so we include them by sharing them out to tax units in a
#     household based on share of dependents younger than 13
#
#  4. there have been concerns previously raised about the validity of the cps
#     variable linking dependents to tax filer (dep_stat) and the number of
#     non-filers/non-dependents with earnings. we have looked at both and in the
#     2017 asec version they appear reasonable but one should proceed with
#     caution.
#
#  5. we produce the rate disaggregated to the state level for convience w/ our
#     application as they are aggregated up to a much more aggregated resolution
#     both in terms of space and households later in our workstream. care should
#     be taken using the rates at the state level, especially for the lowest
#     income household groups.
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

library(tidyr)
library(RCurl)
library(ggplot2)

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","cps_asec.R"))



# ------------------------------------------------------------------------------
# set general parameters
# ------------------------------------------------------------------------------

# if the sage years are not defined, specify a single year
if (!exists("sage.years"))
  sage.years = 2016

# year of the benchmark
year = sage.years[1]

# directory where data files will be stored
data.dir = file.path("build","data")

# upper bounds of the income bins in implan along with the set indexes
hh.inc = data.frame(label=c("hhl","hh15","hh30","hh40","hh50", "hh70","hh100","hh150","hh200"),
                    value=c(15000, 30000, 40000, 50000, 70000, 100000,150000, 200000,    Inf),
                    stringsAsFactors=FALSE)

# gams input file to create
output.file = "tax_data.gms"

# national taxable capital income by type from irs soi 1040 line item totals
# https://www.irs.gov/pub/irs-soi/16inlinecount.pdf
# thousands of dollars
# hard coded as these are only available as a pdf
interest            = 96640233
ordinary.dividends  = 254065327
qualified.dividends = 202033967
prop.income         = 706486715
cap.gains.1040      = 614215455
business.income     = 328082028



# ------------------------------------------------------------------------------
# load the cps asec data
# ------------------------------------------------------------------------------

# download and process the cps march supplement for the year after the benchmark
# year since income questions on the supplement are for the previous year
cpsasec = load.cps.asec(year+1,file.path(data.dir,"cps"))



# ------------------------------------------------------------------------------
# check for clear problems with filestat and dep_stat
# ------------------------------------------------------------------------------

# concerns have previously been raised about the number of non-filers in the cps
# here we check the percentage of non-filer that are not declared as dependents
# and also have income that would suggest that maybe they should be filing

# personal exemption for the tax year under consideration
pexemp = 4050

# standard deduction for head of household for the tax year under consideration
hh.std.deduct = 9300

# standard deduction for married filing jointly for the tax year under
# consideration
mar.std.deduct = 12600

# check the number of dependents also listed as filers - some may be legit, but
# worth checking
print(paste0("Percent of dependents listed as filers: ",
             signif(sum(cpsasec$dep_stat>0 & cpsasec$filestat<6)/
                    sum(cpsasec$dep_stat>0)*100,2),"%"))

# check the number of dependents also listed as filers but with income less
# than the personal exemption - not impossible but a concern if this is large
print(paste0("Percent of dependents listed as filers w/ income less than ",
             "personal exemption: ",
             signif(sum(cpsasec$dep_stat>0 & cpsasec$filestat<6 &
                        cpsasec$agi<pexemp)/
                    sum(cpsasec$dep_stat>0)*100,2),"%"))

# number of non filers that are also not a dependent
non.filer.non.dependent = sum(cpsasec$filestat==6 & cpsasec$dep_stat==0)

# number of single or head of households that are non filers and not a dependent
# and whose income obviously exceeds the threshold for filing - this doesn't
# distinguish between single and head of household so may falsely throw away
# issues for singles
single.issues = sum(cpsasec$a_maritl>3 & cpsasec$filestat==6 &
                    cpsasec$dep_stat==0 &
                    (cpsasec$inc_tot-cpsasec$ss_val-cpsasec$ssi_val)>
                      (pexemp+hh.std.deduct))

# number of married persons that are non filers and not a dependent
# and whose income obviously exceeds the threshold for filing - this assumes
# all married couple file jointly
married.issues = sum(cpsasec$a_maritl<=3 & cpsasec$filestat==6 &
                     cpsasec$dep_stat==0 &
                     (cpsasec$inc_tot-cpsasec$ss_val-cpsasec$ssi_val)>
                       (pexemp*2+mar.std.deduct))

print(paste0("Percent of non-filer non-dependent cases that are clear problems: ",
             signif((married.issues+single.issues)/
                    non.filer.non.dependent,2)*100,"%"))



# ------------------------------------------------------------------------------
# state and year
# ------------------------------------------------------------------------------

# taxsimid  case id (arbitrary, but must be a non-negative numeric)
# just use the row in the cps asec data so we can map it back if needed
cpsasec$taxsimid = 1:nrow(cpsasec)

# yeartax   year
# asec data is for the preceeding year so set the tax year to the previous year
cpsasec$yeartax = cpsasec$h_year-1

# state     soi codes
# add the state soi code to the data using the fips codes
cpsasec$state = factor(cpsasec$gestfips,levels=c( 1L, 2L, 4L, 5L, 6L, 8L, 9L,
                                                  10L,11L,12L,13L,15L,16L,17L,
                                                  18L,19L,20L,21L,22L,23L,24L,
                                                  25L,26L,27L,28L,29L,30L,31L,
                                                  32L,33L,34L,35L,36L,37L,38L,
                                                  39L,40L,41L,42L,44L,45L,46L,
                                                  47L,48L,49L,50L,51L,53L,54L,
                                                  55L,56L),
                       labels=1:51)



# ------------------------------------------------------------------------------
# create marital status and ages
# ------------------------------------------------------------------------------

# mstat     marital status
#             1. single or head of household (unmarried)
#             2. joint (married)
#             6. separate (married)
#             8. dependent taxpayer (typically child with income)
#             head of household status is determined by taxsim27
cpsasec$mstat = 1
cpsasec$mstat[cpsasec$filestat %in% 1:3] = 2
cpsasec$mstat[cpsasec$filestat==5 & cpsasec$dep_stat>0] = 8
cpsasec$mstat[cpsasec$filestat==6] = 0

# make sure that any dependent filing a tax return is not counted as a dependent
# on anyone else's tax return by removing them as a dependent
cpsasec$dep_stat[cpsasec$filestat<6] = 0

# page      age of primary taxpayer
cpsasec$page = cpsasec$a_age

# find the record assocaited with each spouse - doing this once allows for
# faster processing later on when we need reference spouses
cpsasec$id = cpsasec$h_seq*10000+cpsasec$a_lineno
temp = merge(data.frame(id=cpsasec$id),
             data.frame(id=cpsasec$h_seq*10000+cpsasec$a_spouse,
                        x=1:nrow(cpsasec)),
             sort=FALSE,all.x=TRUE)
names(temp)[names(temp)=="x"] = "spouse_row"
cpsasec = merge(cpsasec,temp,all.x=TRUE,sort=FALSE)

# sage      age of spouse (or zero)
cpsasec$sage = cpsasec$a_age[cpsasec$spouse_row]
cpsasec$sage[is.na(cpsasec$sage)] = 0

# make sure that spouse age is populated for anyone filing jointly
if (any(cpsasec$filestat<=3 & cpsasec$sage==0))
  warning("not all spouse ages populated")

# make sure that spouse age is not populated for anyone not filing jointly
if (any(cpsasec$filestat %in% c(4,5) & cpsasec$sage>0,na.rm=TRUE))
  warning("not all married couples filing jointly")

# make sure that spouse age of a spouse mataches the primary's age
if (any(cpsasec$page!=cpsasec$sage[cpsasec$spouse_row],na.rm=TRUE))
  warning("mismatch in spouse ages")



# ------------------------------------------------------------------------------
# get number of dependents by age
# ------------------------------------------------------------------------------

# this is not the most straightforward way to code this process, but using a for
# loop here or sapply is painfully slow

# get the row of the person claiming each dependent and store that with the
# dependent record
temp = merge(data.frame(id=(cpsasec$h_seq*10000+
                            cpsasec$dep_stat)[cpsasec$dep_stat>0]),
             data.frame(id=cpsasec$h_seq*10000+cpsasec$a_lineno,
                        x=1:nrow(cpsasec)),
             sort=FALSE)$x
cpsasec$dep_row = NA
cpsasec$dep_row[cpsasec$dep_stat>0] = temp

# default to zero dependents
cpsasec$depx = 0
cpsasec$dep13 = 0
cpsasec$dep17 = 0
cpsasec$dep18 = 0

# depx      number of dependents (personal exemptions)
temp = aggregate(rep(1,nrow(cpsasec)),by=list(cpsasec$dep_row),sum)
cpsasec$depx[temp$Group.1] = temp$x

# dep13     number of children under 13 (dependent care credit)
dep_row = cpsasec$dep_row[cpsasec$a_age<13]
temp = aggregate(rep(1,length(dep_row)),by=list(dep_row),sum)
cpsasec$dep13[temp$Group.1] = temp$x

# dep17     number of children under 17 (child credit)
dep_row = cpsasec$dep_row[cpsasec$a_age<17]
temp = aggregate(rep(1,length(dep_row)),by=list(dep_row),sum)
cpsasec$dep17[temp$Group.1] = temp$x

# dep18     number of children under 18 (eitc)
dep_row = cpsasec$dep_row[cpsasec$a_age<18]
temp = aggregate(rep(1,length(dep_row)),by=list(dep_row),sum)
cpsasec$dep18[temp$Group.1] = temp$x

# populate dependents for spouses if married and not primary
cpsasec$depx = pmax(cpsasec$depx,cpsasec$depx[cpsasec$spouse_row],na.rm=TRUE)
cpsasec$dep13 = pmax(cpsasec$dep13,cpsasec$dep13[cpsasec$spouse_row],na.rm=TRUE)
cpsasec$dep17 = pmax(cpsasec$dep17,cpsasec$dep17[cpsasec$spouse_row],na.rm=TRUE)
cpsasec$dep18 = pmax(cpsasec$dep18,cpsasec$dep18[cpsasec$spouse_row],na.rm=TRUE)

# verify that dependent count is the same across spouses
vars = c("depx","dep13","dep17","dep18")
if (any(cpsasec[,vars]!=cpsasec[cpsasec$spouse_row,vars],na.rm=TRUE))
  warning("dependent counts not equal across spouses")



# ------------------------------------------------------------------------------
# income
# ------------------------------------------------------------------------------

# pwages    wage and salary income of primary taxpayer (include self-employment)
# note that we subtract off health insurance premiums paid by the employee
# if they have an employer provided plan
cpsasec$pwages = cpsasec$wsal_val+cpsasec$semp_val+cpsasec$frse_val-
                 cpsasec$hiemp*cpsasec$phip_val

# swages    wage and salary income of spouse (include self-employment)
cpsasec$swages = cpsasec$pwages[cpsasec$spouse_row]
cpsasec$swages[is.na(cpsasec$swages)] = 0

# dividends dividend income (qualified dividends only for 2003 on)
# dividends are split between qualified dividends and other property income,
# which includes ordinary dividends, based on 1040 line item totals
cpsasec$dividends = cpsasec$div_val*qualified.dividends/
                    (qualified.dividends+ordinary.dividends)

# intrec    interest received (+/-)
cpsasec$intrec = cpsasec$int_val

# cps stopped imputing capital gains in 2010
# stcg      short term capital gains or losses (+/-)
# ltcg      long term capital gains or losses (+/-)
cpsasec$stcg = 0
cpsasec$ltcg = 0

# otherprop other property income, including unearned partnership and s-corp
#           income, rent, non-qualified dividends, capital gains distributions
#           on form 1040 other income or loss not otherwise enumerated here
cpsasec$otherprop = cpsasec$rnt_val+cpsasec$div_val*ordinary.dividends/
                    (qualified.dividends+ordinary.dividends)

# nonprop   other non-property income (not subject to medicare niit) such as
#           alimony, fellowships, state income tax refunds (itemizers only),
#           adjustments and items such as alimony paid, keogh and ira
#           contributions foreign income exclusion nols can be entered here as
#           negative income (+/-)
cpsasec$nonprop = cpsasec$oi_val+cpsasec$ed_val

# pensions  taxable pensions and ira distributions
cpsasec$pensions = cpsasec$rtm_val

# gssi      gross social security benefits
# include social security (ss), supplemental social security (ssi), survivors
# benefits (srvs), and disability (dsab)
cpsasec$gssi = cpsasec$ss_val+cpsasec$ssi_val+cpsasec$srvs_val+cpsasec$dsab_val

# ui        unemployment compensation received
cpsasec$ui = cpsasec$uc_val

# transfers other non-taxable transfer income such as welfare, workers comp,
#           veterans benefits, child support that would affect eligibility for
#           state property tax rebates but would not be taxable at the federal
#           level
# include public assistance and welfare (paw), workers compensation (wc),
# vetrans benefits (vet), child support (csp), and financial assistance (fin)
cpsasec$transfers = cpsasec$paw_val+cpsasec$wc_val+cpsasec$vet_val+
                    cpsasec$csp_val+cpsasec$fin_val

# add spouses non-earnings income together
vars = c("dividends","intrec","otherprop","nonprop","pensions","gssi","ui",
         "transfers")
for (v in vars) {
  spouse = cpsasec[cpsasec$spouse_row,v]
  spouse[is.na(spouse)] = 0
  cpsasec[,v] = cpsasec[,v]+spouse
}

# make sure that spouse wage of a spouse mataches the primary's wage
if (any(cpsasec$pwage!=cpsasec$swage[cpsasec$spouse_row],na.rm=TRUE))
  warning("mismatch in spouse wages")

# verify that spouses have the same non-wage income
if (any(cpsasec[,vars]!=cpsasec[cpsasec$spouse_row,vars],na.rm=TRUE))
  warning("non-wage income not equal across spouses")



# ------------------------------------------------------------------------------
# get the number of tax filers per household
# ------------------------------------------------------------------------------

# property tax is only available for households so we distribute evenly over tax
# filing units per household - therefore, we need to get the number of distinct
# tax filing unit per household

# aggregate the number of filers per household by filing type
hh_num = aggregate(rep(1,nrow(cpsasec)),
                   by=list(cpsasec$h_seq,cpsasec$filestat),sum)
names(hh_num) = c("h_seq","filestat","count")

# aggregate all joint filers into a single group with code 1
hh_num$filestat[hh_num$filestat<=3] = 1
hh_num = aggregate(hh_num$count,
                   by=list(hh_num$h_seq,hh_num$filestat),sum)
names(hh_num) = c("h_seq","filestat","count")

# divide joing filer count by 2 to get to filing units
hh_num$count[hh_num$filestat==1] = hh_num$count[hh_num$filestat==1]/2

# remove non-filers from final count by zeroing them out
hh_num$count[hh_num$filestat==6] = 0

# final count of filing units per household
hh_num = aggregate(hh_num$count,by=list(hh_num$h_seq),sum)
names(hh_num) = c("h_seq","h_filer_count")
cpsasec = merge(cpsasec,hh_num,all.x=TRUE,sort=FALSE)



# ------------------------------------------------------------------------------
# deductions
# ------------------------------------------------------------------------------

# rentpaid  rent paid (used only for calculating state property tax rebates)
# not available in the cps
cpsasec$rentpaid = 0

# proptax   real estate taxes paid. this is a preference for the amt and is is
#           also used to calculate state property tax rebates
# household value is split evenly between tax filing units
cpsasec$proptax = cpsasec$prop_tax/cpsasec$h_filer_count

# otheritem other itemized deductions that are a preference for the alternative
#           minimum tax. these would include other state and local taxes
#           (line 8 of schedule a) plus local income tax and preference share of
#           medical expenses
cpsasec$otheritem = 0

# childcare child care expenses
# the cps includes this information at the household level but assume 0 and use
# the amount implicitly included in the census imputed deductions instead
cpsasec$childcare = 0

# applicability amounts for the personal exemption phase out
cpsasec$pexemp_cap = 250000*(cpsasec$mstat==8)+
                     275000*(cpsasec$mstat==1)+
                     300000*(cpsasec$mstat==2)

# define the personal exemption amount for each filer including the phase out
cpsasec$pexemp = (cpsasec$depx+1)*pexemp*
                 pmax((1-pmax(cpsasec$agi-cpsasec$pexemp_cap,0)/2500*.02),0)

# mortgage  deductions not included in otheritem and not a preference for the
#           amt, including (on schedule a for 2009), such as deductible medical
#           expenses, motor vehicle taxes paid, home mortgage interest,
#           charitable contributions, casulty or theft losses
# use the census imputation when possible
cpsasec$mortgage = cpsasec$agi-cpsasec$tax_inc-cpsasec$pexemp

# add the standard deductions to the dataset
cpsasec$std_deduct = 6300*(cpsasec$mstat==8)+
                     hh.std.deduct*(cpsasec$mstat==1)+
                     mar.std.deduct*(cpsasec$mstat==2)

# check if there are any cases where the "imputed dedductions" are negative
# (i.e., zero taxable income) but that shouldn't be the case based on personal
# exemptions and standard deduction alone - not impossible but a high value
# here would give concern about the imputed deductions
print(paste0("Percent of zero taxable income filers not based on exemptions ",
             "and standard deductions alone: ",
             signif(sum((cpsasec$agi>cpsasec$std_deduct+cpsasec$pexemp &
                         cpsasec$mortgage<0)[cpsasec$mstat>0])/
                    sum((cpsasec$mortgage<0)[cpsasec$mstat>0]),2)*100,"%"))

# remove salt deductions from potential itemized deductions so they are not
# overestimated
cpsasec$mortgage = cpsasec$mortgage-cpsasec$proptax-cpsasec$statetax_ac

# assume that filers with no apparent taxable income have no itemized deductions
cpsasec$mortgage[cpsasec$mortgage<0] = 0

# populate deductions for spouses if married and not primary
cpsasec$mortgage = pmax(cpsasec$mortgage,cpsasec$mortgage[cpsasec$spouse_row],
                        na.rm=TRUE)



# ------------------------------------------------------------------------------
# function for runing taxsim
# ------------------------------------------------------------------------------

run.taxsim = function(cpsasec,taxsim.version=27,marginal.type=11,
                      detail.level=0) {

  # variables used by taxsim
  taxsim_vars = c("taxsimid","yeartax","state","mstat","page","sage","depx",
                  "dep13","dep17","dep18","pwages","swages","dividends",
                  "intrec","stcg","ltcg","otherprop","nonprop","pensions",
                  "gssi","ui","transfers","rentpaid","proptax","otheritem",
                  "childcare","mortgage")

  # cretate the data set for taxsim
  data = cpsasec[cpsasec$filestat<6,taxsim_vars]

  data$dividends = round(data$dividends)
  data$otherprop = round(data$otherprop)

  # other taxsim options
  options = rep(0,ncol(data))
  options[1] = taxsim.version  # taxsim version
  options[2] = marginal.type   # marginal calculation type
  options[3] = detail.level    # detail level

  # create unique file name for upload to taxsim
  y = regexpr('[0-9][0-9][:punct:][0-9][0-9][:punct:][0-9][0-9]',Sys.time(),TRUE)
  z = y+attr(y,"match.length")-1
  time = unlist(strsplit(substr(Sys.time(),y,z),":"))
  time2 = paste(time,collapse="")
  taxsim.file = paste0(time2,Sys.getpid())

  # write the file to be uploaded to taxsim
  write.table(t(options),taxsim.file,row.names=FALSE,quote=FALSE,sep=",",
              col.names=FALSE)
  write.table(data,taxsim.file,row.names=FALSE,quote=FALSE,sep=",",
              col.names=FALSE,append=TRUE)

  # upload the file to taxsim
  ftpUpload(taxsim.file,
            paste0("ftp://taxsim:02138@taxsimftp.nber.org/tmp/",taxsim.file))

  # retrieve the output from taxsim
  # takes a moment because taxsim process the file once its requested
  # timeout after 30 minutes - should be enough time based on cps size
  out = try(getURLContent(url=paste0("ftp://taxsimftp.nber.org/tmp/",
                                     taxsim.file,".txm27"),
                          userpwd="taxsim:02138",
                          ftp.use.epsv=FALSE,
                          dirlistonly=FALSE,
                          timeout=1800),
            silent=TRUE)

  # if taxsim output could be retrieved load it, else display an error message
  if (grepl("Operation timed out",out,fixed=TRUE)==TRUE) {
    stop(paste0("\nunable to retrieve taxsim output: \n\n",out))
  } else {
    tax.data = read.table(text=out,header=TRUE,stringsAsFactors=FALSE)
  }

  # retrieve any messages about the run from taxsim
  # timeout after 30 second - should be enough time given its just a text file
  error = try(getURLContent(url=paste0("ftp://taxsimftp.nber.org/tmp/",
                                       taxsim.file,".msg"),
                            userpwd="taxsim:02138",
                            ftp.use.epsv=FALSE,
                            dirlistonly=FALSE,
                            timeout=30),
              silent=TRUE)

  # if it could be retrieved display the *.msg file from taxsim which contains
  # any information from taxsim about the run, else display the error message
  if (grepl("Operation timed out",error,fixed=TRUE)==TRUE) {
    warning(paste0("\nunable to retrieve .msg file from taxsim: \n\n",error))
  } else {
    cat(error)
  }

  # remove the taxsim input file
  file.remove(taxsim.file)

  return(tax.data)

}

taxsim.version = function(taxsim.version=27) {

  # create unique file name for upload to taxsim
  y = regexpr('[0-9][0-9][:punct:][0-9][0-9][:punct:][0-9][0-9]',Sys.time(),TRUE)
  z = y+attr(y,"match.length")-1
  time = unlist(strsplit(substr(Sys.time(),y,z),":"))
  time2 = paste(time,collapse="")
  taxsim.file = paste0(time2,Sys.getpid())

  # upload the file to taxsim
  ftp = paste0("ftp://taxsim:02138@taxsimftp.nber.org/tmp/",taxsim.file)
  taxsim.ver = c("NBER/TAXSIM",ftp,taxsim.version)

  return(taxsim.ver)

}

# ------------------------------------------------------------------------------
# save version of taxsim model used for calculations
# ------------------------------------------------------------------------------

tax.vers = taxsim.version(taxsim.version=27)
tax.vers = data.frame(t(tax.vers))
colnames(tax.vers)[1] <- "agency"
colnames(tax.vers)[2] <- "call_id"
colnames(tax.vers)[3] <- "version"
write.table(tax.vers,
            file="build/data/satellite_data_versions.csv",
            append = TRUE,
            sep=',',
            row.names=FALSE,
            col.names=FALSE)



# ------------------------------------------------------------------------------
# earnings weighted effective marginal tl tax rates by state and household type
# ------------------------------------------------------------------------------

# run taxsim for marginal taxes on wages and report version
tax.data = run.taxsim(cpsasec,taxsim.version=27,marginal.type=85,detail.level=0)

# total wages tax rate
tax.data$rate = tax.data$frate+tax.data$srate

# merge additional cps data into the taxsim results
tax.data = merge(tax.data,
                 cpsasec[,c("taxsimid","pwages","marsupwt","htotval",
                            "gestfips")],
                 by.x="taxsim_id",by.y="taxsimid",sort=FALSE)


# average tax rate
tax.data$tl_avg = (tax.data$fiitax+tax.data$siitax)/tax.data$htotval

# adjust the tax rates to account for the fact that taxes are not paid on the
# employers share of fica taxes since they will be multiplied by the full
# compensation in the model, including the employers share
tax.data$rate = tax.data$rate*(1/(1+tax.data$ficar/2/100))
tax.data$tl_avg = tax.data$tl_avg*(1/(1+tax.data$ficar/2/100))
tax.data$ficar = tax.data$ficar*(1/(1+tax.data$ficar/2/100))

# calculate the earnings weighted effective marginal tax rates
# filers with no earnings are given a fica rate of 0 by taxsim even though their
# first dollar of income would be taxed at the statutory rate. this doesn't
# matter for the earnings weighted average but for other averaging methods this
# would need to be taken into consideration
tax.data$tl = tax.data$rate*tax.data$pwages*tax.data$marsupwt
tax.data$tp = tax.data$ficar*tax.data$pwages*tax.data$marsupwt

# calculate the earnings weighted average tax rates. the average tax rate for
# each return is based on total income but the weighting is done via primary
# wages since each joint return show up twice switching the primary and
# secondary wage earners
tax.data$tl_avg = tax.data$tl_avg*tax.data$pwages*tax.data$marsupwt

# add implan household label to each tax unit
bins = c(-Inf,hh.inc$value)
for (i in 1:nrow(hh.inc))
  tax.data$hh[tax.data$htotval>bins[i] &
                tax.data$htotval<=bins[i+1]] = hh.inc$label[i]

# get state average earnings weighted effective marginal tax rates
state.tax = aggregate(tax.data[,c("tl","tp","tl_avg")],
                      by=list(tax.data$gestfips,tax.data$hh),sum)
names(state.tax)[1:2] = c("fips","hh")
state.tax$wgt = aggregate(tax.data$pwages*tax.data$marsupwt,
                          by=list(tax.data$gestfips,tax.data$hh),sum)$x
state.tax$tl = state.tax$tl/state.tax$wgt
state.tax$tp = state.tax$tp/state.tax$wgt
state.tax$tl_avg = state.tax$tl_avg/state.tax$wgt

# add the state abbreviations to the data using the fips codes
state.tax$state = factor(state.tax$fips,levels = c(1L,  2L,  4L,  5L,  6L,
                                                   8L,  9L, 10L, 11L, 12L,
                                                   13L, 15L, 16L, 17L, 18L,
                                                   19L, 20L, 21L, 22L, 23L,
                                                   24L, 25L, 26L, 27L, 28L,
                                                   29L, 30L, 31L, 32L, 33L,
                                                   34L, 35L, 36L, 37L, 38L,
                                                   39L, 40L, 41L, 42L, 44L,
                                                   45L, 46L, 47L, 48L, 49L,
                                                   50L, 51L, 53L, 54L, 55L,
                                                   56L),
                                        labels = c("al","ak","az","ar","ca",
                                                   "co","ct","de","dc","fl",
                                                   "ga","hi","id","il","in",
                                                   "ia","ks","ky","la","me",
                                                   "md","ma","mi","mn","ms",
                                                   "mo","mt","ne","nv","nh",
                                                   "nj","nm","ny","nc","nd",
                                                   "oh","ok","or","pa","ri",
                                                   "sc","sd","tn","tx","ut",
                                                   "vt","va","wa","wv","wi",
                                                   "wy"))

# limit the fica tax rate to the statutory rate to avoid rounding errors
fica.rate = 15.3
state.tax$tp = pmin(state.tax$tp,fica.rate)

# convert to fractions from percentages
state.tax$tl = state.tax$tl/100
state.tax$tp = state.tax$tp/100

# add the payroll taxes in the the wage income taxes
state.tax$tlp = state.tax$tl+state.tax$tp

# write the gams input file
write.gams.variable(file.path(data.dir,output.file),"tl0",
                    state.tax[,c("state","hh","tl")],
                    comments="earnings weighted average effective marginal tax rate on wages")

# write the gams input file
write.gams.variable(file.path(data.dir,output.file),"tfica0",
                    state.tax[,c("state","hh","tp")],
                    comments="earnings weighted average effective fica marginal tax rate",
                    append=TRUE)

# write the gams input file
write.gams.variable(file.path(data.dir,output.file),"tl_avg0",
                    state.tax[,c("state","hh","tl_avg")],
                    comments="earnings weighted average personal income tax rate",
                    append=TRUE)



# ------------------------------------------------------------------------------
# plot the state by household tax rates
# ------------------------------------------------------------------------------

# set the order of the household labels to be increasing in income
state.tax$hh = factor(state.tax$hh,hh.inc$label)

# convert the tax rate to a percentage
state.tax$tl = state.tax$tl*100
state.tax$tlp = state.tax$tlp*100
state.tax$tl_avg = state.tax$tl_avg*100

# plot the variation in wage tax rates by household types across states
p = ggplot(state.tax)+
      geom_boxplot(aes(x=hh,y=tl))+
      scale_y_continuous(breaks=round(seq(-40,60,by=5),1))+
      labs(y="Effective Marignal Income Tax Rate on Wage Earnings",x="Household")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.box.just  = "left",
            legend.key       = element_blank())

print(p)

# plot the variation in wage tax rates by household types across states
# including payroll taxes
p = ggplot(state.tax)+
      geom_boxplot(aes(x=hh,y=tlp))+
      scale_y_continuous(breaks=round(seq(-40,60,by=5),1))+
      labs(y="Effective Marignal Income and Payroll Tax Rate on Wages",
           x="Household")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.box.just  = "left",
            legend.key       = element_blank())

print(p)

# plot the variation in average tax rates by household types across states
p = ggplot(state.tax)+
      geom_boxplot(aes(x=hh,y=tl_avg))+
      scale_y_continuous(breaks=round(seq(-40,60,by=5),1))+
      labs(y="Average Personal Income Tax Rate",x="Household")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.box.just  = "left",
            legend.key       = element_blank())

print(p)



# ------------------------------------------------------------------------------
# earnings weighted effective marginal personal income tax rate on capital
# ------------------------------------------------------------------------------

# run taxsim for marginal taxes on interest income
tax.data.k = run.taxsim(cpsasec,taxsim.version=27,marginal.type=14,
                        detail.level=0)
tax.data.k$int = tax.data.k$frate+tax.data.k$srate

# run taxsim for marginal taxes on qualified dividends
# taxsim doesn't appear to have a margin type for dividends (which are defined
# as qualified in taxsim) so use the long term capital gains margin instead
temp = run.taxsim(cpsasec,taxsim.version=27,marginal.type=70,detail.level=0)
tax.data.k$div = temp$frate+temp$srate

# run taxsim for marginal taxes on other property income (including ordindary
# dividends)
temp = run.taxsim(cpsasec,taxsim.version=27,marginal.type=17,detail.level=0)
tax.data.k$prop = temp$frate+temp$srate

# merge additional cps data into the taxsim results
tax.data.k = merge(tax.data.k,
                   cpsasec[,c("taxsimid","dividends","intrec","marsupwt",
                              "htotval","gestfips","otherprop")],
                   by.x="taxsim_id",by.y="taxsimid",sort=FALSE)

# earnings weighted national interest income tax rate
tk.int = sum(tax.data.k$int*tax.data.k$intrec*tax.data.k$marsupwt)/
         sum(tax.data.k$intrec*tax.data.k$marsupwt)

# earnings weighted national qualified dividend income tax rate
tk.div = sum(tax.data.k$div*tax.data.k$dividends*tax.data.k$marsupwt)/
         sum(tax.data.k$dividends*tax.data.k$marsupwt)

# earnings weighted national property income tax rate
tk.prop = sum(tax.data.k$prop*tax.data.k$otherprop*tax.data.k$marsupwt)/
          sum(tax.data.k$otherprop*tax.data.k$marsupwt)



# ------------------------------------------------------------------------------
# coporate income tax rate
# ------------------------------------------------------------------------------

# tax rate - corporate income tax
# from cbo (2017) international comparison of corporate income tax rates
# https://www.cbo.gov/system/files/115th-congress-2017-2018/reports/52419-internationaltaxratecomp.pdf
tk.corp = 18.6



# ------------------------------------------------------------------------------
# effective marginal tax rate of capital income
# ------------------------------------------------------------------------------

# weighted average for the effective marginal tax rate on capital assuming that
# interest and dividend income is taxed both as corporate income and personal
# income and that sole proprietorship and s corporation income is only tax as
# personal income where weights are based on 1040 line item totals
tk = data.frame(r=unique(state.tax$state),
                value=((tk.corp+tk.int)*interest+
                       (tk.corp+tk.div)*qualified.dividends+
                       (tk.corp+tk.prop)*ordinary.dividends+
                       tk.prop*(prop.income+cap.gains.1040+business.income))/
                      (interest+ordinary.dividends+qualified.dividends+
                       prop.income+cap.gains.1040+business.income)/100)

# write the gams input file for the toal capital tax
write.gams.variable(file.path(data.dir,output.file),"tk0",
                    tk,
                    comments="capital income tax rate",
                    append=TRUE)



# ------------------------------------------------------------------------------
# payroll tax
# ------------------------------------------------------------------------------

# tax rate - payroll tax
# not used as the earnings weighted effective marginal fica tax rate by
# household is included seperately as tfica
tp = data.frame(r=unique(state.tax$state),value=0)

# write the gams input file
write.gams.variable(file.path(data.dir,output.file),"tp0",
                    tp,
                    comments="payroll tax rate",
                    append=TRUE,
                    write.zeros=TRUE)



# ------------------------------------------------------------------------------
# consumption (i.e., sales) tax
# ------------------------------------------------------------------------------

# tax rate - consumption
# not used as sales and excise taxes are included in the implan production tax
# rates
tc = data.frame(r=unique(state.tax$state),value=0)

# write the gams input file
write.gams.variable(file.path(data.dir,output.file),"tc0",
                    tc,
                    comments="consumption tax rate",
                    append=TRUE,
                    write.zeros=TRUE)
