# baseline_visualization.R
#



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# directory to store the plots
if (!exists("image.dir"))
  image.dir = file.path("output","baseline_plots")

# plot device and resolution
if (!exists("plot.type"))
  plot.type = "pdf"
if (!exists("plot.dpi"))
  plot.dpi = 150

# benchmark file to use in sage simulation
if (!exists("benchmark_file"))
  benchmark_file = "default_aggregation.gdx"

# should the model be run or rely on the existing output/baseline.csv results
if (!exists("run_sage"))
  run_sage = TRUE

# when presenting growth rates across different periods use these ranges
growth.periods = data.frame(start=c(2021,2036),end=c(2036,2051))

# end year for the plots over time
plot.end.year = 2051

# baseline file
baseline_file = "baseline.gms"



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# define functions and other preliminaries
# ------------------------------------------------------------------------------

# get the value (p*q) for a given parameter and price combination
get.value = function(parameter,price="pa") {

  # extract the price
  p = bau[bau$parameter==price,c("time","region","sector","value")]
  names(p)[names(p)=="value"] = "p"

  # remove columns that don't have any meaning for the given price
  if (p$region[1]=="")
    p = p[,names(p)!="region"]
  if (p$sector[1]=="")
    p = p[,names(p)!="sector"]

  # merge the price with the requested parameter
  value = merge(bau[bau$parameter==parameter,],p)

  # calculate the value of the requested parameter
  value$value = value$value*value$p
  value = value[,names(value)!="p"]

  return(value)

}

# convert value data to real terms using the cpi
get.real = function(data) {

  # extract the price
  p = bau[bau$parameter=="cpi",c("time","value")]
  names(p)[names(p)=="value"] = "p"

  # merge the price with the data
  value = merge(data,p)

  # calculate the real present value of the requested parameter
  value$value = value$value/value$p
  value = value[,names(value)!="p"]

  return(value)

}

# convert value data to share of real gdp
get.gdp.share = function(data) {

  # extract gdp
  p = bau[bau$parameter=="gdp",c("time","value")]
  names(p)[names(p)=="value"] = "p"

  # merge gdp with the data
  value = merge(data,p)

  # calculate the share of gdp
  value$value = value$value/value$p
  value = value[,names(value)!="p"]

  return(value)

}

# get average growth rates over specified periods
get.period.growth = function(data) {
  growth = data.frame(period=paste0(growth.periods$start,"-",
                                    growth.periods$end),
                      value=NA,
                      stringsAsFactors=FALSE)
  for (i in 1:nrow(growth.periods))
    growth$value[i] = (data$value[data$time==growth.periods$end[i]]/
                       data$value[data$time==growth.periods$start[i]])^
                      (1/(growth.periods$end[i]-growth.periods$start[i]))-1

  return(growth)

}

# create output directory if it doesn't exist
if (!dir.exists(image.dir))
  dir.create(image.dir)



# ------------------------------------------------------------------------------
# run the model
# ------------------------------------------------------------------------------

# run the baseline
if (run_sage) {
  start_time = Sys.time()
  run.model(output_file="baseline.csv",
            benchmark_file=benchmark_file)
  end_time = Sys.time()

  # display runtime for the baseline
  cat(paste0("\nruntime for the baseline: ",round(end_time-start_time,1),"\n\n"))
}

# load the results
bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)



# ------------------------------------------------------------------------------
# growth rates for macroeconomic variables
# ------------------------------------------------------------------------------

# add sage real gdp grwoth rate
temp = bau[bau$parameter=="gdp_real",]
plot.data = data.frame(t=temp$time[-1],
                       value=(temp$value[-1]/temp$value[-nrow(temp)])^
                               (1/(temp$time[-1]-temp$time[-nrow(temp)]))-1,
                       variable="Real GDP")

# add capital stock growth to the plot data
temp = combine.new.extant(bau,"k")
temp = aggregate(list(value=temp$value),by=list(time=temp$time),sum)
plot.data = rbind(plot.data,
                 data.frame(t=temp$time[-1],
                            value=(temp$value[-1]/temp$value[-nrow(temp)])^
                                    (1/(temp$time[-1]-temp$time[-nrow(temp)]))-1,
                            variable="Capital Stock"))

# labor supply in sage
ld = aggregate(list(value=bau$value[bau$parameter=="ld"]),
               by=list(t=bau$time[bau$parameter=="ld"]),sum)
ld$value = ld$value+aggregate(list(value=bau$value[bau$parameter=="ld_ex"]),
                              by=list(t=bau$time[bau$parameter=="ld_ex"]),
                              sum)$value

# add labor supply growth to the plot data
plot.data = rbind(plot.data,
                  data.frame(t=ld$t[-1],
                             value=(ld$value[-1]/ld$value[-nrow(ld)])^
                                    (1/(ld$t[-1]-ld$t[-nrow(ld)]))-1,
                             variable="Labor Supply"))

# sage labor productivity
temp = bau[bau$parameter=="gdp_real",]
temp$value = temp$value/ld$value

# add labor productivity growth to the plot data
plot.data = rbind(plot.data,
                  data.frame(t=temp$time[-1],
                             value=(temp$value[-1]/temp$value[-nrow(temp)])^
                                     (1/(temp$time[-1]-temp$time[-nrow(temp)]))-1,
                             variable="Labor Productivity"))

# add population growth to the plot data
temp = bau[bau$parameter=="n",]
temp = aggregate(list(value=temp$value),by=list(time=temp$time),sum)
plot.data = rbind(plot.data,
                  data.frame(t=temp$time[-1],
                             value=(temp$value[-1]/temp$value[-nrow(temp)])^
                                     (1/(temp$time[-1]-temp$time[-nrow(temp)]))-1,
                             variable="Labor Force"))

# convert to percent
plot.data$value = plot.data$value*100

# set ordering for plot
plot.data$variable = factor(plot.data$variable,
                            c("Real GDP","Capital Stock","Labor Productivity",
                              "Labor Supply","Labor Force"))

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$variable))]

p = ggplot(plot.data)+
      geom_line(aes(x=t,y=value,linetype=variable,color=variable),size=1)+
      labs(y="Annual Growth [%]",
           x="Year")+
      scale_x_continuous(limits=c(min(plot.data$t),plot.end.year))+
      scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                     "twodash"))+
      scale_color_manual(values=colors)+
      guides(linetype=guide_legend(title="Variable",
                                   override.aes=list(color=colors)),
             color="none")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("macro_growth_rates.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# comparison of sage to cbo aggregate labor productivity growth
# ------------------------------------------------------------------------------

# load the baseline file - so we can pull the calibration points
baseline = readLines(file.path("data",baseline_file))

# extract the aggregate productivity index
line = baseline[grep("l_prod_agg",baseline)]
lp.agg = data.frame(t=as.numeric(substr(line,regexpr('[(]',line)+2,
                                      regexpr('")',line)-1)),
                        value=as.numeric(substr(line,regexpr('=',line)+1,
                                                     regexpr(';',line)-1)))

# convert the aggregate productivity index to an average annual growth rate
lp.agg$value[-1] = ((lp.agg$value[-1]/lp.agg$value[-nrow(lp.agg)])^
                               (1/(lp.agg$t[-1]-lp.agg$t[-nrow(lp.agg)]))-1)*100
lp.agg$value[1] = NA
lp.agg$variable = "CBO"

# pull out the labor productivity growth from the previous plot data
plot.data = plot.data[plot.data$variable=="Labor Productivity",]
plot.data$variable = "SAGE"

# merge the SAGE baseline and CBO calibration points
plot.data = rbind(plot.data,lp.agg)

# set ordering for plot
plot.data$variable = factor(plot.data$variable,c("SAGE","CBO"))

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$variable))]

p = ggplot(plot.data)+
      geom_line(aes(x=t,y=value,linetype=variable,color=variable),size=1)+
      labs(y="Annual Growth [%]",
           x="Year")+
      scale_x_continuous(limits=c(min(plot.data$t),plot.end.year))+
      scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                     "twodash"))+
      scale_color_manual(values=colors)+
      guides(linetype=guide_legend(title="Source",
                                   override.aes=list(color=colors)),
             color="none")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("prod_agg_compare.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# comparison of sage sectoral productivity inputs compared to historical
# ------------------------------------------------------------------------------

# extract the sector specific productivity index from the baseline file
line = baseline[grep("l_prod",baseline)]
lp = data.frame(time=as.numeric(substr(line,regexpr('[(]',line)+2,
                                            regexpr('[(]',line)+5)),
                sector=substr(line,regexpr('[(]',line)+9,regexpr('")',line)-1),
                value=as.numeric(substr(line,regexpr('=',line)+1,
                                             regexpr(';',line)-1)))
lp = lp[lp$sector!="",]

# get the growth rates by sector and period
plot.data = NULL
for (s in unique(lp$sector)) {
  growth = get.period.growth(lp[lp$sector==s,])
  growth$sector = s
  plot.data = rbind(plot.data,growth)
}

temp = read.csv(file.path("documentation","hist_prod_growth.csv"),
                stringsAsFactors=FALSE)
temp$period = "Historical"

periods = unique(plot.data$period)
plot.data = rbind(plot.data,temp[,c("period","value","sector")])

# convert to percent
plot.data$value = plot.data$value*100

# order for plotting (since the axis are flipped these need to be in reverse
# order)
plot.data$period = factor(plot.data$period,c(rev(periods),"Historical"))
temp = plot.data[plot.data$period==unique(plot.data$period)[1],]
plot.data$sector = factor(plot.data$sector,
                          temp$sector[sort(temp$value,index.return=TRUE,
                                           decreasing=TRUE)$ix])

p = ggplot(plot.data)+
     geom_bar(aes(x=sector,y=value,fill=period),stat="identity",
              position="dodge")+
     labs(y="Annual Labor Productivity Growth [%]",
          x="Sector")+
     coord_flip()+
     scale_fill_manual(values=sage.palette)+
     guides(fill=guide_legend(title="Period",reverse=TRUE))+
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(.75,"cm"),
           legend.position  = c(.8,.5),
           strip.background = element_blank())

print(p)

ggsave(file.path(image.dir,paste0("sector_prod_compare.",plot.type)),
       device=plot.type,width=8,height=8,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# unit enery consumption index for firms
# ------------------------------------------------------------------------------

# extract the sector specific productivity index from the baseline file
line = baseline[grep("ene_growth",baseline)]
ene = data.frame(time=as.numeric(substr(line,regexpr('[(]',line)+2,
                                             regexpr('[(]',line)+5)),
                 sen=substr(line,regexpr('[(]',line)+9,regexpr('[(]',line)+11),
                 sector=substr(line,regexpr('[(]',line)+15,regexpr('")',line)-1),
                 value=as.numeric(substr(line,regexpr('=',line)+1,
                                              regexpr(';',line)-1)))
ene = ene[ene$sector!="",]

ene = ene[ene$sen=="ele",c("time","sector","value")]

# get the growth rates by sector and period
plot.data = NULL
for (s in unique(ene$sector)) {
 growth = get.period.growth(ene[ene$sector==s,])
 growth$sector = s
 plot.data = rbind(plot.data,growth)
}

# convert to percent
plot.data$value = plot.data$value*100

# order for plotting (since the axis are flipped these need to be in reverse
# order)
plot.data$period = factor(plot.data$period,c(rev(unique(plot.data$period))))
temp = plot.data[plot.data$period==unique(plot.data$period)[1],]
plot.data$sector = factor(plot.data$sector,
                         temp$sector[sort(temp$value,index.return=TRUE,
                                          decreasing=TRUE)$ix])

p = ggplot(plot.data)+
    geom_bar(aes(x=sector,y=value,fill=period),stat="identity",
             position="dodge")+
    labs(y="Annual Unit Energy Consumption Growth [%]",
         x="Sector")+
    coord_flip()+
    scale_fill_manual(values=sage.palette)+
    guides(fill=guide_legend(title="Period",reverse=TRUE))+
    theme(axis.line.x      = element_line(colour="black"),
          axis.line.y      = element_line(colour="black"),
          axis.ticks.y     = element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border     = element_blank(),
          panel.background = element_blank(),
          legend.key       = element_blank(),
          legend.key.width = unit(.75,"cm"),
          legend.position  = c(.2,.5),
          strip.background = element_blank())

print(p)

ggsave(file.path(image.dir,paste0("ene_growth.",plot.type)),
      device=plot.type,width=8,height=8,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# unit enery consumption index for households
# ------------------------------------------------------------------------------

# extract the sector specific productivity index from the baseline file
line = baseline[grep("cd_ene_growth",baseline)]
ene = data.frame(time=as.numeric(substr(line,regexpr('[(]',line)+2,
                                             regexpr('[(]',line)+5)),
                 sector=substr(line,regexpr('[(]',line)+9,
                                    regexpr('[(]',line)+11),
                 value=as.numeric(substr(line,regexpr('=',line)+1,
                                              regexpr(';',line)-1)))

# remove coal since it isn't used by households
ene = ene[ene$sector!="col",]

# get the growth rates by sector and period
plot.data = NULL
for (s in unique(ene$sector)) {
 growth = get.period.growth(ene[ene$sector==s,])
 growth$sector = s
 plot.data = rbind(plot.data,growth)
}

# convert to percent
plot.data$value = plot.data$value*100

# order for plotting (since the axis are flipped these need to be in reverse
# order)
plot.data$period = factor(plot.data$period,c(rev(unique(plot.data$period))))
temp = plot.data[plot.data$period==unique(plot.data$period)[1],]
plot.data$sector = factor(plot.data$sector,
                         temp$sector[sort(temp$value,index.return=TRUE,
                                          decreasing=TRUE)$ix])

p = ggplot(plot.data)+
    geom_bar(aes(x=sector,y=value,fill=period),stat="identity",
             position="dodge")+
    labs(y="Annual Unit Energy Consumption Growth [%]",
         x="Sector")+
    coord_flip()+
    scale_fill_manual(values=sage.palette)+
    guides(fill=guide_legend(title="Period",reverse=TRUE))+
    theme(axis.line.x      = element_line(colour="black"),
          axis.line.y      = element_line(colour="black"),
          axis.ticks.y     = element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border     = element_blank(),
          panel.background = element_blank(),
          legend.key       = element_blank(),
          legend.key.width = unit(.75,"cm"),
          legend.position  = c(.2,.5),
          strip.background = element_blank())

print(p)

ggsave(file.path(image.dir,paste0("cd_ene_growth.",plot.type)),
      device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# row income and transfer gdp shares over time
# ------------------------------------------------------------------------------

# years of historic data to plot
years = 1990:2019

# create a container for the plot data
plot.data = NULL

# function to return a zero if there is no value present
sz = function(x) if (length(x)==0) { return(0) } else { return(x) }

# for each of the historic years grab the bea nipa data
for (year in years) {

  # get the foreign transactions nipa table
  f_table = get.bea.variable("T40100",year)

  # name of the column with the values
  col = names(f_table)==paste0("DataValue_",year)

  # get nominal gdp from the bea nipa tables
  gdp = get.bea.variable("T10105",year,row=1)[1]

  # net income from rest of the world excluding government interest payments
  # which are handled separately in sage
  inc_row = (sz(f_table[f_table$LineNumber==7,col])-
             sz(f_table[f_table$LineNumber==23,col])+
             get.bea.variable("T30100",year,row=29)[1])/gdp

  # net taxes and transfers from the rest of the world
  tran_row = (sz(f_table[f_table$LineNumber==13,col])-
              sz(f_table[f_table$LineNumber==29,col]))/gdp

  # save the current year's data
  plot.data = rbind(plot.data,
                    data.frame(time=year,
                               Variable=c("Net Income","Net Transfers"),
                               value=c(inc_row,tran_row)))

}

# convert to a percent
plot.data$value = plot.data$value*100

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$Variable))]

p = ggplot(plot.data)+
      geom_line(aes(x=time,y=value,linetype=Variable,color=Variable),size=1)+
      labs(y="Percent of GDP [%]",
           x="Year")+
      scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                     "twodash"))+
      scale_color_manual(values=colors)+
      guides(linetype=guide_legend(title="Account",
                                   override.aes=list(color=colors)),
             color="none")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("hist_row_accounts.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# plot gdp accounts from expenditure side
# ------------------------------------------------------------------------------

plot.data = get.value("cd")
plot.data = rbind(plot.data,get.value("i"))
plot.data = rbind(plot.data,get.value("g"))
plot.data = rbind(plot.data,get.value("x_f",price="pfx"))
plot.data = rbind(plot.data,get.value("m_f",price="pfx"))

plot.data$parameter[plot.data$parameter=="cd"] = "Consumption"
plot.data$parameter[plot.data$parameter=="i"] = "Investment"
plot.data$parameter[plot.data$parameter=="g"] = "Government"
plot.data$parameter[plot.data$parameter=="x_f"] = "Exports"
plot.data$parameter[plot.data$parameter=="m_f"] = "Imports"

plot.data = get.gdp.share(plot.data)
plot.data$value = plot.data$value*100

plot.data = aggregate(list(value=plot.data$value),
                      by=list(time=plot.data$time,
                              parameter=plot.data$parameter),sum)

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$parameter))]

p = ggplot(plot.data)+
      geom_line(aes(x=time,y=value,linetype=parameter,color=parameter),size=1)+
      labs(y="Percent of GDP [%]",
           x="Year")+
      scale_x_continuous(limits=c(min(plot.data$time),plot.end.year))+
      scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                     "twodash"))+
      scale_color_manual(values=colors)+
      guides(linetype=guide_legend(title="Account",
                                   override.aes=list(color=colors)),
             color="none")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("gdp_accounts.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# plot government account variables
# ------------------------------------------------------------------------------

plot.data = get.value("g")
temp = get.value("deficit",price="cpi")
temp$value = -temp$value
plot.data = rbind(plot.data,temp)
temp = get.value("gint",price="cpi")
temp$value = temp$value+get.value("gint_row",price="pfx")$value
plot.data = rbind(plot.data,temp)
plot.data = rbind(plot.data,get.value("transfers",price="cpi"))
plot.data = rbind(plot.data,bau[bau$parameter=="tax_revenue",])

plot.data$parameter[plot.data$parameter=="g"] = "Gov. Consumption"
plot.data$parameter[plot.data$parameter=="deficit"] = "Deficit"
plot.data$parameter[plot.data$parameter=="gint"] = "Interest"
plot.data$parameter[plot.data$parameter=="transfers"] = "Transfers"
plot.data$parameter[plot.data$parameter=="tax_revenue"] = "Tax Revenue"

plot.data = get.gdp.share(plot.data)
plot.data$value = plot.data$value*100

plot.data = aggregate(list(value=plot.data$value),
                      by=list(time=plot.data$time,
                              parameter=plot.data$parameter),sum)

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$parameter))]

p = ggplot(plot.data)+
     geom_line(aes(x=time,y=value,linetype=parameter,color=parameter),size=1)+
     labs(y="Percent of GDP [%]",
          x="Year")+
     scale_x_continuous(limits=c(min(plot.data$time),plot.end.year))+
     scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                    "twodash"))+
     scale_color_manual(values=colors)+
     guides(linetype=guide_legend(title="Account",
                                  override.aes=list(color=colors)),
            color="none")+
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("government_accounts.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# plot foreign account variables
# ------------------------------------------------------------------------------

plot.data = get.value("")
plot.data = rbind(plot.data,get.value("curactbal",price="pfx"))
plot.data = rbind(plot.data,get.value("tran_row",price="pfx"))
plot.data = rbind(plot.data,get.value("inc_row",price="pfx"))
plot.data = rbind(plot.data,get.value("gint_row",price="pfx"))
#plot.data = rbind(plot.data,get.value("m_f",price="pfx"))
#plot.data = rbind(plot.data,get.value("x_f",price="pfx"))
temp = get.value("x_f",price="pfx")
temp$value = temp$value-get.value("m_f",price="pfx")$value
plot.data = rbind(plot.data,temp)

plot.data$parameter[plot.data$parameter=="curactbal"] = "Current Account Bal."
plot.data$parameter[plot.data$parameter=="tran_row"] = "Net Transfers"
plot.data$parameter[plot.data$parameter=="inc_row"] = "Net Income"
plot.data$parameter[plot.data$parameter=="gint_row"] = "Gov. Interest"
plot.data$parameter[plot.data$parameter=="x_f"] = "Net Exports"

plot.data = get.gdp.share(plot.data)
plot.data$value = plot.data$value*100

plot.data = aggregate(list(value=plot.data$value),
                      by=list(time=plot.data$time,
                              parameter=plot.data$parameter),sum)

# colors vector for the plot based on the number of variables being plotted
colors = sage.palette[1:length(unique(plot.data$parameter))]

p = ggplot(plot.data)+
     geom_line(aes(x=time,y=value,linetype=parameter,color=parameter),size=1)+
     labs(y="Percent of GDP [%]",
          x="Year")+
     scale_x_continuous(limits=c(min(plot.data$time),plot.end.year))+
     scale_linetype_manual(values=c("solid","dashed","dotdash","dotted",
                                    "twodash"))+
     scale_color_manual(values=colors)+
     guides(linetype=guide_legend(title="Account",
                                  override.aes=list(color=colors)),
            color="none")+
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(1.25,"cm"))

print(p)

ggsave(file.path(image.dir,paste0("foreign_accounts.",plot.type)),
       device=plot.type,width=8,height=4.5,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# average secotral output growth
# ------------------------------------------------------------------------------

# baseline output aggregated across vintages
y = combine.new.extant(bau,"y")

# aggregate across regions
y = aggregate(list(value=y$value),by=list(time=y$time,sector=y$sector),sum)

# get the growth rates
plot.data = NULL
for (s in unique(y$sector)) {
  growth = get.period.growth(y[y$sector==s,])
  growth$sector = s
  plot.data = rbind(plot.data,growth)
}

# convert to percent
plot.data$value = plot.data$value*100

# order for plotting (since the axis are flipped these need to be in reverse
# order)
plot.data$period = factor(plot.data$period,rev(unique(plot.data$period)))
temp = plot.data[plot.data$period==unique(plot.data$period)[1],]
plot.data$sector = factor(plot.data$sector,
                          temp$sector[sort(temp$value,index.return=TRUE,
                                           decreasing=TRUE)$ix])

p = ggplot(plot.data)+
     geom_bar(aes(x=sector,y=value,fill=period),stat="identity",
              position="dodge")+
     labs(y="Annualized Output Growth [%]",
          x="Sector")+
     coord_flip()+
     scale_fill_manual(values=sage.palette)+
     guides(fill=guide_legend(title="Period",reverse=TRUE))+
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(.75,"cm"),
           strip.background = element_blank(),
           legend.position  = c(.7,.5))

print(p)

ggsave(file.path(image.dir,paste0("sector_output_growth.",plot.type)),
       device=plot.type,width=4,height=8,units="in",dpi=plot.dpi)



# ------------------------------------------------------------------------------
# average sector price growth
# ------------------------------------------------------------------------------

# baseline real national prices
pn = get.real(bau[bau$parameter=="pn",])

# get the growth rates
plot.data = NULL
for (s in unique(pn$sector)) {
  growth = get.period.growth(pn[pn$sector==s,])
  growth$sector = s
  plot.data = rbind(plot.data,growth)
}

# convert to percent
plot.data$value = plot.data$value*100

# order for plotting (since the axis are flipped these need to be in reverse
# order)
plot.data$period = factor(plot.data$period,rev(unique(plot.data$period)))
#temp = plot.data[plot.data$period==unique(plot.data$period)[1],]
plot.data$sector = factor(plot.data$sector,
                          temp$sector[sort(temp$value,index.return=TRUE,
                                           decreasing=TRUE)$ix])

p = ggplot(plot.data)+
     geom_bar(aes(x=sector,y=value,fill=period),stat="identity",
              position="dodge")+
     labs(y="Annualized Price Growth [%]",
          x="Sector")+
     coord_flip()+
     scale_fill_manual(values=sage.palette)+
     guides(fill="none")+
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(.75,"cm"),
           strip.background = element_blank())

print(p)

ggsave(file.path(image.dir,paste0("sector_price_growth.",plot.type)),
       device=plot.type,width=4,height=8,units="in",dpi=plot.dpi)
