# income_elasticity.R
#
# simulates the income elasticity of consumption and plots the results over
# households and time against the calibration values
#
# output:
#  output/income_elasticity_results.csv
#
# dependencies:
#  examples/income_elasticity.gms
#



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# use the default aggregation benchmark file
benchmark_file = "default_aggregation.gdx"

# parameter file
parameter_file = "parameters.gms"

# size of the text in the plot
text_size = 12



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the icnome elasticity simulations
# ------------------------------------------------------------------------------

# run the income elasticity simulations
run.model(benchmark_file=benchmark_file,
          parameter_file=parameter_file,
          options="--epilogue=diagnostics////income_elasticity.gms")

# load the results of the simulations
ie = read.csv(file.path("output","income_elasticity_results.csv"),
              stringsAsFactors=FALSE)



# ------------------------------------------------------------------------------
# plot the results of the simulations
# ------------------------------------------------------------------------------

# read in the calibration values for the income elasticities
calelast = read.csv(file.path("build","data",
                              "default_aggregation_income_elasticities.csv"),
                     stringsAsFactors=FALSE)
calelast = subset(calelast,type %in% c("cal"))
calelast = subset(calelast,household %in% c("hh5"))
calelast = calelast[,!(names(calelast) %in% c("type","household"))]
names(calelast)[names(calelast)=="value"] = "target"

# merge the calibration targets and the implicit elasticities
ie = merge(ie,calelast)

# remove rows for which there is no consumption of the commodity
ie = ie[ie$target!=1 & ie$value!=0,]

# plot the simulation results
p = ggplot(ie)+
      geom_hline(aes(yintercept=target),linetype=2)+
      geom_jitter(aes(household,value,color=time),size=1.5,height=0)+
      facet_wrap(.~sector)+
      labs(x="Household",y="Income Elasticity")+
      guides(color=guide_legend(title="Year",override.aes=list(size=3)))+
      scale_color_distiller(palette="RdYlBu")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            strip.background = element_blank())
print(p)
