# labor_supply_elasticity.R
#
# simulates the substitution and income effect in the labor supply elasticity
# and plots the results over households and time against the calibration values
#
# output:
#  output/lse_results.csv
#
# dependencies:
#  examples/labor_supply_elasticity.gms
#



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# use the default aggregation benchmark file
benchmark_file = "default_aggregation.gdx"

# parameter file
parameter_file = "parameters.gms"

# size of the text in the plot
text_size = 12



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the labor supply elasticity simulations
# ------------------------------------------------------------------------------

# run the labor supply elasticity simulations
run.model(benchmark_file=benchmark_file,
          parameter_file=parameter_file,
          options="--epilogue=diagnostics////labor_supply_elasticity.gms")

# load the results of the simulations
lse = read.csv(file.path("output","lse_results.csv"),stringsAsFactors=FALSE)



# ------------------------------------------------------------------------------
# plot the results of the simulations
# ------------------------------------------------------------------------------

# load the parameters file - so we can pull the calibration points
parameters = readLines(file.path("data",parameter_file))

# get the calibration point for the substitution effect
line = parameters[grep("lse_sub",parameters)]
lse_sub = as.numeric(substr(line,regexpr("=",line)[1]+1,regexpr(";",line)[1]-1))

# get the calibration point for the income effect
line = parameters[grep("lse_inc",parameters)]
lse_inc = as.numeric(substr(line,regexpr("=",line)[1]+1,regexpr(";",line)[1]-1))

# calculate the income effect
lse$value[lse$variable=="uncomp"] = lse$value[lse$variable=="uncomp"]-
                                    lse$value[lse$variable=="comp"]

# rename the variables to match the type of effect
lse$variable[lse$variable=="comp"] = "Substitution"
lse$variable[lse$variable=="uncomp"] = "Income"

# plot the simulation results
p = ggplot(lse,aes(household,value,color=time,shape=variable))+
      geom_hline(aes(yintercept=lse_sub),linetype=2)+
      geom_hline(aes(yintercept=lse_inc),linetype=2)+
      geom_jitter(size=2,height=0)+
      scale_y_continuous(limits=c(-.1,.25))+
      labs(x="Household",y="Labor Supply Elasticity")+
      guides(shape=guide_legend(title="Effect"),
             color=guide_legend(title="Year"))+
      scale_color_distiller(palette="RdYlBu")+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.position  = c(.89,.5))
print(p)
