# numeraire_test.r
#
# This program checcks the homogeneity assumptions of the model. Here, we
# verify that if we perturb the model's numeraire by some perturbation factor
# and compare a baseline with the numeraire set to 1 and a baseline generated
# with a numeraire set to (1+perturb), then all prices and value variables
# change by the same perturbation factor in the latter baseline relative to
# the former baseline, but quantity variables are unchanged. Note that fixed
# prices in baseline will be fixed to the same value in both baselines
# irrespective of the value chosen for the numeraire (when a particular
# activity is zero).

setwd("e:/git/CGE")

# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","install_R_packages.R"))

library(ggplot2)
library(dplyr)



# ------------------------------------------------------------------------------
# construct numeraire test
# ------------------------------------------------------------------------------

# perturbation factor
perturb = 0.2

# generate a "policy" file that changes the numeraire
delta_numeraire = "numeraire_test.gms"
fc = file(file.path("diagnostics",delta_numeraire),open="wt")
writeLines(paste0("pfx.fx(t)$tfirst(t) = (1+",perturb,")*1;"),con=fc)
close(fc)



# ------------------------------------------------------------------------------
# run the model
# ------------------------------------------------------------------------------

# set parameters file to run test for a given dynamic assumption
parameters = "parameters.gms"

# run the baseline
run.model(output_file="baseline_numeraire_one.csv",parameter_file=parameters)

# run the baseline with a change in numeraire from 1 to (1+perturb)
run.model(output_file="baseline_numeraire_delta.csv",
          parameter_file=parameters,
          policy_file=paste0("diagnostics",get.path.sep(),delta_numeraire))



# ------------------------------------------------------------------------------
# compare baseline variables
# ------------------------------------------------------------------------------

# load the results
bau = read.csv(file.path("output","baseline_numeraire_one.csv"),
               stringsAsFactors=FALSE)
names(bau)[7] <- "bau"
bau_delta = read.csv(file.path("output","baseline_numeraire_delta.csv"),
                     stringsAsFactors=FALSE)
names(bau_delta)[7] <- "bau_delta"

# net out prices that are fixed across baselines. to do so, difference the
# varibales before aggregating.
prices = c("pkt","cpi","pr_ex_agg","pc","r","kh","tax_revenue","pa","gdp",
           "pcl","pn","pgov","pl","pfx","pd","py","pr_ex","pk","pr","pres",
           "prh","prh_ex","pm","px")
diff = left_join(bau, bau_delta,
                 by=c("parameter","time","region","sector","household","sector.from"))
diff$x = diff$bau_delta - diff$bau
diff.prices = subset(diff, !(diff$x==0) & diff$parameter %in% prices)
diff.vars = subset(diff, !(diff$parameter %in% prices))
diff = rbind(diff.prices, diff.vars)

# calculate the percentage change in the selected variables
diff = data.frame(diff %>%
    group_by(parameter) %>%
    summarise_at(c("bau","bau_delta"),.funs=sum))
diff$pct = (diff$bau_delta/diff$bau - 1)*100

# remove NaN entries
diff = na.omit(diff)

# reorder variables by percent change
diff$parameter <- factor(diff$parameter, levels = diff$parameter[order(diff$pct)])

# plot the change in variables
p = ggplot(diff, aes(x=parameter, y=pct))+
    geom_jitter(position = position_jitter(0),
                colour="#9C9F84",
                cex=2, shape=15) +
    coord_flip() +
    ggtitle(paste0("Percent Change in Baseline Variables: Numeraire Pertubed by ",
                   perturb)) +
    labs(y="Change in variable across baselines [%]",
         x="Variable Name")+
    theme(axis.line.x      = element_line(colour="black"),
          axis.line.y      = element_line(colour="black"),
          axis.ticks.y     = element_blank(),
          panel.grid.major.x = element_blank(),
          panel.grid.major.y = element_line(size=.1, color="gray"),
          panel.grid.minor = element_blank(),
          panel.border     = element_blank(),
          panel.background = element_blank(),
          legend.position  = "none")
print(p)
