# plots_for_documentation.R
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","install_R_packages.R"))

library(ggplot2)
library(xtable)



# ------------------------------------------------------------------------------
# set general options
# ------------------------------------------------------------------------------

# directory to store the images
image.dir = file.path("documentation")

# type of image format in which to save plots
plot.type = "pdf"

# dpi of saved plots
plot.dpi = 150

# benchmark file
benchmark_file = "default_aggregation.gdx"

# parameter file
parameter_file = "parameters.gms"



# ------------------------------------------------------------------------------
# region map
# ------------------------------------------------------------------------------

plot.sage.regions(aggregation.file="default_aggregation.gms")

ggsave(file.path(image.dir,paste0("regions.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=4.5)



# ------------------------------------------------------------------------------
# labor supply elasticity calibration
# ------------------------------------------------------------------------------

# run the labor supply elasticity simulations
source(file.path("diagnostics","labor_supply_elasticity.R"))

# save the plot
ggsave(file.path(image.dir,paste0("lse_results.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=4.5)



# ------------------------------------------------------------------------------
# main substitution elasticity table
# ------------------------------------------------------------------------------

# load the parameters file - so we can pull the calibration points
parameters = readLines(file.path("data",parameter_file))

# elasticities to include in the table
elasticities = c("se_kl","se_kle","se_klem","se_ene","se_en","se_nf","se_dn")

# build the elasticity table from the parameter file
for (parameter in elasticities) {
  line = parameters[grep(paste0(parameter,"\\("),parameters)]
  temp = data.frame(s=substr(line,regexpr('[(]',line)+2,
                             regexpr('")',line)-1),
                    stringsAsFactors=FALSE)
  temp[,parameter] = as.numeric(substr(line,regexpr('=',line)+1,
                                       regexpr(';',line)-1))
  if (parameter==elasticities[1])
    se = temp
  else
    se = merge(se,temp)
}

# write the latex table with the parameters
of = file(file.path("documentation","se_table.tex"),open="w")
writeChar("\\begin{tabular}{c",con=of,eos=NULL)
for (i in 1:length(elasticities))
  writeChar("c",con=of,eos=NULL)
writeLines("}\\\\",con=of)
writeLines("\\hline",con=of)
writeChar("Sector",con=of,eos=NULL)
for (i in 1:length(elasticities))
  writeChar(paste0(" & ",gsub("_","\\\\_",elasticities[i])),con=of,eos=NULL)
writeLines(" \\\\",con=of)
writeLines("\\hline",con=of)
for (i in 1:nrow(se)) {
  writeChar(as.character(se$s[i]),con=of,eos=NULL)
  for (j in 1:length(elasticities))
    writeChar(paste0(" & ",format(round(se[i,j+1],2),nsmall=2)),con=of,eos=NULL)
  writeLines(" \\\\",con=of)
}
writeLines("\\hline",con=of)
writeLines("\\end{tabular}",con=of)
close(of)



# ------------------------------------------------------------------------------
# ene_growth mapping table
# ------------------------------------------------------------------------------

# load the nems to sage mapping
nems.to.sage = read.csv(file.path("build","data","nems_to_sage.csv"),
                        stringsAsFactors=FALSE)

# write the latex table of the mapping
of = file(file.path("documentation","ene_growth_table.tex"),open="w")
writeLines("\\begin{tabular}{ll}",con=of)
writeLines("\\hline",con=of)
writeLines("SAGE & AEO \\\\",con=of)
writeLines("Sector & Sectors \\\\",con=of)
writeLines("\\hline",con=of)
for (s in unique(nems.to.sage[,1])) {
  line = paste0(s," & ")
  nems = nems.to.sage[nems.to.sage[,1]==s,2]
  for (i in 1:length(nems)) {
    if (i==1)
      line = paste0(line,nems[i])
    else
      line = paste0(line,", ",nems[i])
  }
  writeLines(paste0(line," \\\\"),con=of)
}
writeLines("\\hline",con=of)
writeLines("\\end{tabular}",con=of)
close(of)



# ------------------------------------------------------------------------------
# cd_ene_growth table
# ------------------------------------------------------------------------------

# get the values for ene_growth from the parameter file
line = parameters[grep("cd_ene_growth",parameters)]
ene_growth = data.frame(s=substr(line,regexpr('[(]',line)+2,
                                      regexpr('")',line)-1),
                        value=as.numeric(substr(line,regexpr('=',line)+1,
                                                     regexpr(';',line)-1)))

# write the latex table of the mapping
of = file(file.path("documentation","cd_ene_growth_table.tex"),open="w")
writeLines("\\begin{tabular}{ll}",con=of)
writeLines("\\hline",con=of)
writeLines("Commodity & $cd\\_ene\\_growth$ \\\\",con=of)
writeLines("\\hline",con=of)
for (i in 1:nrow(ene_growth)) {
  writeLines(paste0(ene_growth$s[i]," & ",round(ene_growth$value[i],4)," \\\\"),
             con=of)
}
writeLines("\\hline",con=of)
writeLines("\\end{tabular}",con=of)
close(of)



# ------------------------------------------------------------------------------
# write baseline parameters to individual files
# ------------------------------------------------------------------------------

# values to write
names = c("rbar","delta")

# write the parameters to individual files
for (parameter in names) {
  line = parameters[grep(parameter,parameters)]
  temp = substr(line,regexpr('=',line)+1,regexpr(';',line)-1)
  fc = file(file.path("documentation",paste0(parameter,".tex")))
  writeLines(as.character(round(as.numeric(temp),3)),con=fc)
  close(fc)
}



# ------------------------------------------------------------------------------
# create tables based on sam data
# ------------------------------------------------------------------------------

# run the model baseline with the table creatation code
run.model(benchmark_file=benchmark_file,
          parameter_file=parameter_file,
          output_file="baseline.csv",
          options=" --epilogue=documentation////create_tables.gms")

# load the baseline results
bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)



# ------------------------------------------------------------------------------
# plot household labor tax rates
# ------------------------------------------------------------------------------

# get the labor tax rate
tl = bau[bau$parameter=="tl" & bau$time==min(bau$time,na.rm=TRUE),]

# get the sage regions and order the same as in the region map plot
regions = get.sage.regions()
tl$region = factor(tl$region,rev(regions))

# get the min/max rates for the crossbars on the plot
bars = aggregate(tl$value,by=list(tl$household),min)
names(bars) = c("household","min")
bars$max = aggregate(tl$value,by=list(tl$household),max)$x

# get the factor prices by region in the benchmark year
pl = bau[bau$parameter=="pl" & bau$time==min(bau$time,na.rm=TRUE),]
names(pl)[names(pl)=="value"] = "pl"
temp = merge(tl,pl[,c("region","pl")],all.x=TRUE)

# get the factor levels by region in the benchmark year
l = bau[bau$parameter=="l" & bau$time==min(bau$time,na.rm=TRUE),]
names(l)[names(l)=="value"] = "l"
temp = merge(temp,l[,c("region","household","l")],all.x=TRUE)

# compute the return weighted mean tax rate by household in the benchmark
temp$plxl = temp$pl*temp$l
temp$tlxplxl = temp$value*temp$pl*temp$l
temp$mean = NA
for (i in 1:nrow(temp)) {
  ind = temp$household==temp$household[i]
  temp$mean[i] = sum(temp$tlxplxl[ind])/sum(temp$plxl[ind])
}
bars = merge(bars,unique(temp[,c("household","mean")]))

ggplot()+
  geom_crossbar(data=bars,aes(x=household,y=mean,ymin=min,ymax=max),width=.7,
                color="grey")+
  geom_jitter(data=tl,aes(x=household,y=value,color=region),height=0,width=.3,
              size=5)+
  labs(x="Household",y=bquote("Labor Income Tax Rate - "~tl["r,h"]))+
  guides(color=FALSE)+
  scale_color_manual(values=sage.palette)+
  theme(text             = element_text(size=12),
        axis.line.x      = element_line(colour="black"),
        axis.line.y      = element_line(colour="black"),
        axis.ticks.y     = element_blank(),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border     = element_blank(),
        panel.background = element_blank(),
        legend.key       = element_blank(),
        legend.position  = c(.89,.3))

# save the plot
ggsave(file.path(image.dir,paste0("tl.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=4.5)



# ------------------------------------------------------------------------------
# plot household fica tax rates
# ------------------------------------------------------------------------------

# get the labor tax rate
tfica = bau[bau$parameter=="tfica" & bau$time==min(bau$time,na.rm=TRUE),]

# get the sage regions and order the same as in the region map plot
regions = get.sage.regions()
tfica$region = factor(tfica$region,rev(regions))

# get the min/max rates for the crossbars on the plot
bars = aggregate(tfica$value,by=list(tfica$household),min)
names(bars) = c("household","min")
bars$max = aggregate(tfica$value,by=list(tfica$household),max)$x

# get the factor prices by region in the benchmark year
pl = bau[bau$parameter=="pl" & bau$time==min(bau$time,na.rm=TRUE),]
names(pl)[names(pl)=="value"] = "pl"
temp = merge(tfica,pl[,c("region","pl")],all.x=TRUE)

# get the factor levels by region in the benchmark year
l = bau[bau$parameter=="l" & bau$time==min(bau$time,na.rm=TRUE),]
names(l)[names(l)=="value"] = "l"
temp = merge(temp,l[,c("region","household","l")],all.x=TRUE)

# compute the return weighted mean tax rate by household in the benchmark
temp$plxl = temp$pl*temp$l
temp$tlxplxl = temp$value*temp$pl*temp$l
temp$mean = NA
for (i in 1:nrow(temp)) {
  ind = temp$household==temp$household[i]
  temp$mean[i] = sum(temp$tlxplxl[ind])/sum(temp$plxl[ind])
}
bars = merge(bars,unique(temp[,c("household","mean")]))

ggplot()+
  geom_crossbar(data=bars,aes(x=household,y=mean,ymin=min,ymax=max),width=.7,
                color="grey")+
  geom_jitter(data=tfica,aes(x=household,y=value,color=region),height=0,
              width=.3,size=5)+
  labs(x="Household",y=bquote("FICA Tax Rate - "~tfica["r,h"]))+
  scale_color_manual(values=sage.palette,
                     labels=rev(get.sage.region.names()$label),
                     name=NULL)+
  theme(text             = element_text(size=12),
        axis.line.x      = element_line(colour="black"),
        axis.line.y      = element_line(colour="black"),
        axis.ticks.y     = element_blank(),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border     = element_blank(),
        panel.background = element_blank(),
        legend.key       = element_blank(),
        legend.position  = c(.2,.4))

# save the plot
ggsave(file.path(image.dir,paste0("tfica.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=4.5)



# ------------------------------------------------------------------------------
# plot household average labor tax rates
# ------------------------------------------------------------------------------

# get the labor tax rate
tl_avg = bau[bau$parameter=="tl_avg",]

# get the sage regions and order the same as in the region map plot
regions = get.sage.regions()
tl_avg$region = factor(tl_avg$region,rev(regions))

# get the min/max rates for the crossbars on the plot
bars = aggregate(tl_avg$value,by=list(tl_avg$household),min)
names(bars) = c("household","min")
bars$max = aggregate(tl_avg$value,by=list(tl_avg$household),max)$x

# get the factor prices by region in the benchmark year
pl = bau[bau$parameter=="pl" & bau$time==min(bau$time,na.rm=TRUE),]
names(pl)[names(pl)=="value"] = "pl"
temp = merge(tl_avg,pl[,c("region","pl")],all.x=TRUE)

# get the factor levels by region in the benchmark year
l = bau[bau$parameter=="l" & bau$time==min(bau$time,na.rm=TRUE),]
names(l)[names(l)=="value"] = "l"
temp = merge(temp,l[,c("region","household","l")],all.x=TRUE)

# compute the return weighted mean tax rate by household in the benchmark
temp$plxl = temp$pl*temp$l
temp$tlxplxl = temp$value*temp$pl*temp$l
temp$mean = NA
for (i in 1:nrow(temp)) {
  ind = temp$household==temp$household[i]
  temp$mean[i] = sum(temp$tlxplxl[ind])/sum(temp$plxl[ind])
}
bars = merge(bars,unique(temp[,c("household","mean")]))

ggplot()+
  geom_crossbar(data=bars,aes(x=household,y=mean,ymin=min,ymax=max),width=.7,
                color="grey")+
  geom_jitter(data=tl_avg,aes(x=household,y=value,color=region),height=0,
              width=.3,size=5)+
  labs(x="Household",y=bquote("Average Labor Income Tax Rate - "~tl_avg0["r,h"]))+
  guides(color=FALSE)+
  scale_color_manual(values=sage.palette)+
  theme(text             = element_text(size=12),
        axis.line.x      = element_line(colour="black"),
        axis.line.y      = element_line(colour="black"),
        axis.ticks.y     = element_blank(),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        panel.border     = element_blank(),
        panel.background = element_blank(),
        legend.key       = element_blank(),
        legend.position  = c(.89,.3))

# save the plot
ggsave(file.path(image.dir,paste0("tl_avg.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=4.5)



# ------------------------------------------------------------------------------
# baseline visualization
# ------------------------------------------------------------------------------

# don't rerun the model when ploting the baseline
run_sage = FALSE

source(file.path("diagnostics","baseline_visualization.R"))



# ------------------------------------------------------------------------------
# shock type example plots and ev tables
# ------------------------------------------------------------------------------

# run the shock type sample simulations
source(file.path("examples","regulatory_modeling_approach.R"))

# save the output change plot
print(p1)
ggsave(file.path(image.dir,paste0("output_change.",plot.type)),
       device=plot.type,dpi=plot.dpi,width=8,height=4)

# save the labor demand plot
print(p2)
ggsave(file.path(image.dir,paste0("labor_demand.",plot.type)),
       device=plot.type,dpi=plot.dpi,width=8,height=4)

# write the ev results table
of = file(file.path("documentation","shock_type_ev_table.tex"),open="w")
writeLines("\\begin{tabular}{ccc}",con=of)
writeLines("\\hline",con=of)
writeLines("Affected & Productivity & Abatement \\\\",con=of)
writeLines("Sources & Shock & Requirement \\\\",con=of)
writeLines("\\hline",con=of)
for (i in 1:nrow(ev.prod.tot))
  writeLines(paste0(ev.prod.tot$Sources[i]," & ",round(ev.prod.tot$value[i],1),
                    " & ",round(ev.abate.tot$value[i],1),"\\\\"),con=of)
writeLines("\\hline",con=of)
writeLines("\\end{tabular}",con=of)
close(of)


# ------------------------------------------------------------------------------
# income elasticities
# ------------------------------------------------------------------------------

## read in estimated elasticities
aguiar <- read.csv(file.path("build","data","national_income_elasticities_CEX_2013_2017.csv"),
                   stringsAsFactors=FALSE)
aguiar <- aguiar[,c("cex","elast")]

## relabel categories
aguiar$cex[aguiar$cex=="alcbev"] <- "Alcoholic Beverages"
aguiar$cex[aguiar$cex=="food"] <- "Food and Other Beverages"
aguiar$cex[aguiar$cex=="tobacc"] <- "Tobacco Products"
aguiar$cex[aguiar$cex=="appar"] <- "Clothing and Apparel"
aguiar$cex[aguiar$cex=="persca"] <- "Personal Care"
aguiar$cex[aguiar$cex=="read"] <- "Reading"
aguiar$cex[aguiar$cex=="educa"] <- "Education"
aguiar$cex[aguiar$cex=="medical"] <- "Medical Treatment"
aguiar$cex[aguiar$cex=="entert"] <- "Entertainment"
aguiar$cex[aguiar$cex=="elec"] <- "Electricity Utilities"
aguiar$cex[aguiar$cex=="natural_gas"] <- "Natural Gas Utilities"
aguiar$cex[aguiar$cex=="fuel_oil"] <- "Heating Fuels"
aguiar$cex[aguiar$cex=="telephone"] <- "Telephone"
aguiar$cex[aguiar$cex=="water"] <- "Water Utilities"
aguiar$cex[aguiar$cex=="shelter"] <- "Housing"
aguiar$cex[aguiar$cex=="oper_furnish"] <- "Housing Supplies and Furnishings"
aguiar$cex[aguiar$cex=="vacation_home_q"] <- "Vacation Home Rentals"
aguiar$cex[aguiar$cex=="tran_fuel"] <- "Transportation Fuels"
aguiar$cex[aguiar$cex=="veh_mainten"] <- "Vehicle Maintenance"
aguiar$cex[aguiar$cex=="veh_finance"] <- "Vehicle Financing"
aguiar$cex[aguiar$cex=="veh_serv2"] <- "Vehicle Services"

## read in linear combination for SAGE sectors
calelast <- read.csv(file.path("build","data","default_aggregation_income_elasticities.csv"),
                     stringsAsFactors=FALSE)
mappedelast <- subset(calelast, type %in% "data")
mappedelast <- unique(mappedelast[,c("sector","value")])
mappedelast <- subset(mappedelast, !(sector %in% c("col","cru","con")))
mappedelast <- rbind(mappedelast, data.frame(sector=NA,value=NA))

# save table w/ income elasticities
report <- cbind(aguiar,mappedelast)
report$elast <- round(report$elast,2)
report$value <- round(report$value,2)
of = file(file.path("documentation",paste0("income_elasticities.tex")),open="w")
for (i in 1:nrow(report)) {
  for (j in 1:ncol(report)) {
    if (is.numeric(report[i,j])) {
      writeLines(as.character(signif(report[i,j],3)),con=of,sep=" ")
    } else {
      writeLines(as.character(report[i,j]),con=of,sep=" ")
    }
    if (j<ncol(report)) {
      writeLines(" & ",con=of,sep=" ")
    } else {
      writeLines(" \\\\",con=of)
    }
  }
}
close(of)

# run the income elasticity simulations
source(file.path("diagnostics","income_elasticity.R"))

# save the plot
ggsave(file.path(image.dir,paste0("ie_results.",plot.type)),device=plot.type,
       dpi=plot.dpi,width=8,height=7)



# ------------------------------------------------------------------------------
# production tax sensitivity simulation
# ------------------------------------------------------------------------------

# run the shock type sample simulations
source(file.path("examples","consumption_tax.r"))

# save the output change plot
print(p1)
ggsave(file.path(image.dir,paste0("consumption_tax_costs.",plot.type)),
       device=plot.type,dpi=plot.dpi,width=8,height=6)



# ------------------------------------------------------------------------------
# large vs. small open economy simulation
# ------------------------------------------------------------------------------

## run the open economy example
source(file.path("examples","open_economy.r"))


# ------------------------------------------------------------------------------
# report large open economy elasticities
# ------------------------------------------------------------------------------

## perturb model for trade elasticities
source(file.path("diagnostics","trade_elasticity.r"))
