# consumption_tax.r
#
# sensitivty simulation illustrating implementing sector specific consumption
# taxes. simulations here will augment consumption taxes in each sector to
# achive an additional 10 billion in government revenue. plot ev outcomes of
# these model runs.



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","install_R_packages.R"))

library(ggplot2)
library(dplyr)



# ------------------------------------------------------------------------------
# construct model runs
# ------------------------------------------------------------------------------

# shock size - billions of benchmark year dollars
shock.size = 10

# benchmark file
benchmark_file = "default_aggregation.gdx"

# benchmark file
parameter_file = "parameters.gms"

# mapping file associated with the benchmark
map.file = "default_aggregation.gms"

# get the list of sectors
sectors = get.sage.sectors(map.file)

# construct policy file for tax perturbation
write.productivity.shock = function(shock.size,sector) {

  # create a gams input file containing the policy shock
  fc = file(file.path("examples","tax_definition.gms"),open="wt")
  writeLines(paste0('s_tax(\"',sector,'\") = YES;'),con=fc)
  writeLines(paste0('shock_size = ',shock.size,';'),con=fc)
  writeLines('',con=fc)
  close(fc)

}

run.experiments = function(options="",shock.size=0.1,
                           benchmark_file="default_aggregation.gdx",
                           parameter_file="parameters.gms",
                           baseline_file="baseline.gms") {

  # run the baseline
  run.model(benchmark_file=benchmark_file,output_file="baseline.csv",
            options=options,
            parameter_file=parameter_file,
            baseline_file=baseline_file)
  bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)

  # name of output file with the model results
  output_file = "cost_experiments.csv"

  # initialize container for the results
  results = NULL

  # loop over each sector in the model and shock it
  for (i in 1:length(sectors)) {

    # current sector
    sector = sectors[i]

    # write the policy file for the sector
    sector.shock.size = write.productivity.shock(shock.size,
                                                 sector)

    # run the policy scenario
    # note that "sector" is not a real option for the model, if there is an
    # error in the solve r prints out a warning message with the system call
    # and this way we can identify which sector had the error
    print(paste0("running ge model for ",sector,"..."))
    run.model(benchmark_file=benchmark_file,
              output_file=output_file,
              policy_file="examples/consumption_tax.gms",
              parameter_file=parameter_file,
              gdx_baseline_file="baseline.gdx",
              gdx_save=0,
              baseline_file=baseline_file)

    # pause to allow gams to fully release the csv file with the results
    #Sys.sleep(1.5)

    # load the results
    ge.policy = read.csv(file.path("output",output_file),stringsAsFactors=FALSE)

    # extract the welfare change
    temp = ge.policy[ge.policy$parameter=="ev",c("region","household","value")]

    # save the aggregate welfare change
    temp = data.frame(sector=sector,value=sum(temp$value),
                      stringsAsFactors=FALSE)

    temp$sector = sector
    results = rbind(results,temp)

  }

  return(results)

}


# ------------------------------------------------------------------------------
# run the model
# ------------------------------------------------------------------------------

# run experiments
results = run.experiments(shock.size=shock.size,
                          benchmark_file=benchmark_file,
                          parameter_file=parameter_file)

results = results[!(results$sector %in% c("col","cru","con")),]

# plot results
p1 = ggplot(results, aes(x=reorder(sector,-value),y=value)) +
     geom_line(color="grey",group=1) +
     geom_point(cex=5, shape=21, color="black", fill="#9C9F84") +
     geom_hline(yintercept=0, color="grey") +
     coord_flip() +
     labs(y="Billions [$] (Equivalent Variation)",x="Sectors") +
     theme(axis.line.x      = element_line(colour="black"),
           axis.line.y      = element_line(colour="black"),
           axis.ticks.y     = element_blank(),
           panel.grid.major = element_blank(),
           panel.grid.minor = element_blank(),
           panel.border     = element_blank(),
           panel.background = element_blank(),
           legend.key       = element_blank(),
           legend.key.width = unit(1.25,"cm"))
print(p1)
