## open_economy.R
##
## program compares welfare estimates with a small open economy relative to a
## large open economy for a simple shock
##
## this example is intended to be run from the top level directory of the model
##



## ------------------------------------------------------------------------------
## load libraries
## ------------------------------------------------------------------------------

## load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
library(ggplot2)
library(dplyr)
library(tidyr)


## ------------------------------------------------------------------------------
## set general options
## ------------------------------------------------------------------------------

## directory to store the images
image.dir = file.path("documentation")

## type of image format in which to save plots
plot.type = "pdf"

## dpi of saved plots
plot.dpi = 150


## ------------------------------------------------------------------------------
## illustrative regulation scenario, run the soe model
## ------------------------------------------------------------------------------

## run the small open economy baseline
run.model(output_file="baseline_soe.csv", options="--loe=0")

## run the policy scenario
run.model(gdx_baseline_file="baseline_soe.gdx",output_file="policy_soe.csv",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"),
          options="--loe=0")


## ------------------------------------------------------------------------------
## illustrative regulation scenario, run the loe model
## ------------------------------------------------------------------------------

## run the large open economy baseline
run.model(output_file="baseline_loe.csv", options="--loe=1")

## run the policy scenario
run.model(gdx_baseline_file="baseline_loe.gdx",output_file="policy_loe.csv",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"),
          options="--loe=1")


## ------------------------------------------------------------------------------
## load results
## ------------------------------------------------------------------------------

## load the soe results
bau.soe <- read.csv(file.path("output","baseline_soe.csv"),stringsAsFactors=FALSE)
names(bau.soe)[7] <- "bau"
bau.soe$trade <- "soe"
policy.soe <- read.csv(file.path("output","policy_soe.csv"),stringsAsFactors=FALSE)
names(policy.soe)[7] <- "policy"
policy.soe$trade <- "soe"

## load the loe results
bau.loe <- read.csv(file.path("output","baseline_loe.csv"),stringsAsFactors=FALSE)
names(bau.loe)[7] <- "bau"
bau.loe$trade <- "loe"
policy.loe <- read.csv(file.path("output","policy_loe.csv"),stringsAsFactors=FALSE)
names(policy.loe)[7] <- "policy"
policy.loe$trade <- "loe"

## collapse
bau <- rbind(bau.soe, bau.loe)
policy <- rbind(policy.soe, policy.loe)
results <- left_join(policy, bau,
                     by=c("parameter","time","region","sector","household","sector.from","trade"))

## calculate deltas and %ch
results$delta <- results$policy - results$bau
results$pct <- 100 * (results$policy/results$bau - 1)


## ------------------------------------------------------------------------------
## calculate ev
## ------------------------------------------------------------------------------

## compare ev by region and households
ev <- subset(results, parameter %in% c("ev_inf"))
ev.tot <- data.frame(ev %>% group_by(trade) %>% summarise(policy = sum(policy)))
ev.tot <- data.frame(ev.tot %>% spread(trade,policy))
names(ev.tot)[1] <- "Large Open"
names(ev.tot)[2] <- "Small Open"

## print aggregate ev changes
of = file(file.path(image.dir,paste0("ev_loe_vs_soe.tex")),open="w")
writeLines("\\begin{tabular}{cc}",con=of)
writeLines("\\hline",con=of)
writeLines(paste0(names(ev.tot)[1], "&", names(ev.tot)[2]," \\\\"),con=of)
writeLines("Economy & Economy \\",con=of)
writeLines("\\hline",con=of)
writeLines(paste0(signif(ev.tot[1,1],4), "&", signif(ev.tot[1,2],4)," \\\\"),
           con=of)
writeLines("\\hline",con=of)
writeLines("\\end{tabular}",con=of)
close(of)



## ------------------------------------------------------------------------------
## calculate changes in imports and exports
## ------------------------------------------------------------------------------

trd <- subset(results, parameter %in% c("x_f","m_f"))
trd.sec <- trd %>% group_by(parameter,sector,trade) %>% summarise(policy=sum(policy), bau=sum(bau))
trd.sec <- data.frame(trd.sec)
trd.sec$pct <- 100 * (trd.sec$policy / trd.sec$bau - 1)
trd.sec$sector <- factor(trd.sec$sector)
trd.sec$parameter <- factor(trd.sec$parameter)
trd.sec$trade <- factor(trd.sec$trade)

## rename elements for plotting
levels(trd.sec$trade) <- c("LOE","SOE")
names(trd.sec)[3] <- "Trade"
levels(trd.sec$parameter) <- c("Imports","Exports")

plot <- ggplot(trd.sec, aes(x=sector, y=pct, fill=Trade))
plot +
  geom_bar(stat="identity", position=position_dodge(), colour="black") +
  labs(y="% change", x="Sector") +
  scale_fill_manual(values=sage.palette) +
  theme_bw() +
  coord_flip() +
  geom_hline(yintercept=0) +
  theme(axis.text.x = element_text(color="black"),
        axis.text.y = element_text(color="black"),
        axis.title.x = element_text(color="black"),
        axis.title.y = element_text(color="black"),
        strip.text = element_text(color="black"),
        legend.text = element_text(color="black"),
        legend.title = element_text(color="black"),
        panel.border = element_rect(colour = "black"),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        axis.line = element_line(colour = "black")) +
  facet_wrap(~parameter, ncol=2, scales="free")
ggsave(file.path(image.dir,paste0("trade_loe_vs_soe.",plot.type)),
       device=plot.type,dpi=plot.dpi,width=8,height=8)
