## plot_results.r
##
## this program illustrates two functions written in R_utilities.R. the first
## produces a .csv pivot table for reporting results. the function,
## generate.pivot, relies on existence of baseline file and a set of policy
## files. the pivot report is only produced for disaggregate results. the
## second function, plot.variable, allows the user to plot either in a line or
## bar graph aggregated results. this plotting function relies on only a single
## baseline file and a single policy file. for plotting sensitivities across
## policy, use the pivot reporter or a custom code.

## here is a list of included variables reported in sage.gms that can be
## reported below.
##  "a"              "bopdef"         "c"              "cd"
##  "cl"             "cl_base"        "cl_base_annual" "cl_base_inf"
##  "cpi"            "d"              "ev"             "ev_annual"
##  "ev_inf"         "exp_base"       "exp_diff"       "g"
##  "gdp"            "i"              "id"             "id_abate"
##  "id_abate_ex"    "id_ex"          "incadj"         "k"
##  "kd"             "kd_abate"       "kd_abate_ex"    "kd_ex"
##  "kh"             "kt"             "l"              "ld"
##  "ld_abate"       "ld_abate_ex"    "ld_ex"          "leisure"
##  "m_d"            "m_f"            "n"              "pa"
##  "pc"             "pcl"            "pd"             "pfx"
##  "pgov"           "pk"             "pkt"            "pl"
##  "pn"             "pr"             "pr_ex"          "pr_ex_agg"
##  "pres"           "py"             "r"              "r_ex"
##  "res"            "res_ex"         "se_cl"          "se_rklem"
##  "tax_revenue"    "tc"             "te"             "tfica"
##  "tk"             "tl"             "tl_avg"         "tl_refund"
##  "tp"             "tran"           "ty"             "w"
##  "x_d"            "x_f"            "y"              "y_ex"


## ------------------------------------------------------------------------------
## load libraries
## ------------------------------------------------------------------------------

setwd("~/git/CGE")

## load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))
source(file.path("utilities","install_R_packages.R"))

## load plotting packages
library(ggplot2)
library(dplyr)


# ------------------------------------------------------------------------------
# run the model akin to basic_example.R
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline.csv")

# run the policy scenario
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))


## ------------------------------------------------------------------------------
## produce pivot table
## ------------------------------------------------------------------------------

## note that this routine can construct a pivot table with a variety of policy
## files, all reliant on the same baseline. this function does not work with
## multiple baselines. for use with multiple policy files, set:
## policyfiles=c("policy1","policy2",...)

## the generate.pivot function outputs a data.frame that includes the value,
## absolute change (delta) and percent change (pctch) between the policy cases
## and the baseline. results are included for all disaggregated entries
## outputted by sage.gms.

pivot <- generate.pivot(baselinefile="baseline",policyfiles=c("policy"))
write.csv(pivot,"output/pivot.csv",row.names=FALSE)


## ------------------------------------------------------------------------------
## plot series of results
## ------------------------------------------------------------------------------

## note that this function, plot.variable, only constructs a simple aggregation
## that sums disaggregated results. this works well for quantities. however, it
## does not allow the user to construct weighted value/price variables. for
## instance, this function does not allow for prices to be weighted by the cpi
## or quantities in aggregation. here, one can at most plot three
## indices. indices not added to the function will be aggregated. set the
## xaxis, legend dimension and faceted dimension. these three dimensions are
## illustrated below. the format (fmt) of outputted statistics include baseline
## or policy levels, absolute difference between policy and baseline levels
## (delta) and percent change between policy and baseline levels (pctch).

## options for plotting
##   - baselinefile = "name of baseline results file (without file extension)"
##   - policyfile = "name of policy results file (without file extension)"
##   - fmt = baseline, policy, pctch, delta
##   - graphtype = line, bar
##   - parameter.dim = see variable list above
##   - xaxis.dim = "", time, region, sector, household, sector.from
##   - legend.dim = "", time, region, sector, household, sector.from
##   - facet.dim = "", time, region, sector, household, sector.from

## to illustrate this plotting function, we show modeled results for consumer
## demand from the basic_example model runs:

## aggregate percent change by household
cd.aggregreate <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="pctch",
                                graphtype="bar", parameter.dim="cd", xaxis.dim="household")

## aggregate percent change by household-region
cd.hr <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="pctch",
                       graphtype="bar", parameter.dim="cd",
                       xaxis.dim="household", facet.dim="region")

## percent change by household-region-time
cd.hrt <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="pctch",
                        graphtype="line", parameter.dim="cd",
                        xaxis.dim="time", legend.dim="household", facet.dim="region")

## aggregate consumer demand baseline level
cd.base <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="baseline",
                         graphtype="line", parameter.dim="cd",
                         xaxis.dim="time", legend.dim="household")

## aggregate consumer demand policy level
cd.poli <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="policy",
                         graphtype="line", parameter.dim="cd",
                         xaxis.dim="time", legend.dim="household")

## aggregate consumer demand absolute change
cd.delt <- plot.variable(baselinefile="baseline", policyfile="policy", fmt="delta",
                         graphtype="line", parameter.dim="cd",
                         xaxis.dim="time", legend.dim="household")

