# policy_phase_in_example.R
#
# example of how to implement a policy that is phased in over time. this script
# runs two different versions of the same policy. one in which the policy
# becomes effective in the second time period of the model and one in which it
# is phased such that the engineering costs in the second modeling period are
# 50% of their full value. in the third modeling period and thereafter, the two
# policies are the same.
#
# the hypothetical regulation is assumed to effect the primary metal
# manufacturing sector and have an engineering cost estiamte that compliance
# will require an additional 0.4% of baseline capital expenditures and 2% of
# labor expenditures when fully implemented. in the second "phased in" policy
# this is a simplified scenario to demonstrate the difference in how a phased
# in policy would be coded.
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the model
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline.csv")

# run the policy scenario
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))

# run the phased in policy scenario
run.model(gdx_baseline_file="baseline.gdx",output_file="policy_phased.csv",
         policy_file=paste0("examples",get.path.sep(),
                            "sample_phased_in_abatement_requirement.gms"))



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# load the results
bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)
policy = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
policy_phased = read.csv(file.path("output","policy_phased.csv"),
                         stringsAsFactors=FALSE)

# list of parameters to output results for
vars = c("gdp","cd","leisure")

get_diff = function(policy,baseline,vars,scenario=NA) {
  ind = bau$parameter %in% vars
  diff = aggregate(bau$value[ind],by=list(bau$parameter[ind],bau$time[ind]),sum)
  diff$x = (aggregate(policy$value[ind],
                      by=list(policy$parameter[ind],policy$time[ind]),sum)$x/
            diff$x-1)*100
  if (!is.na(scenario))
    diff$scenario = scenario
  return(diff)
}

# calculate the percentage change in the selected variables
diff = get_diff(policy,baseline,vars,"Immediate")
diff = rbind(diff,get_diff(policy_phased,baseline,vars,"Phased"))

# plot the change in variables
p = ggplot(diff)+
      geom_line(aes(x=Group.2,y=x,color=Group.1,linetype=scenario))+
      labs(y="Change in Variable [%]",
           x="Year")+
      scale_linetype_manual(values=c("solid","dashed","dotted"))+
      scale_color_manual(values=sage.palette)+
      guides(linetype=guide_legend(title="Scenario"),
             color=guide_legend(title="Variable"))+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank(),
            legend.key.width = unit(1.25,"cm"))

print(p)

# output equivalent variation
ev = sum(policy$value[policy$parameter=="ev_inf"])
writeLines(paste0("EV Immediate..: ",ev))
ev = sum(policy_phased$value[policy_phased$parameter=="ev_inf"])
writeLines(paste0("EV Phased.....: ",ev))
