# regulatory_modeling_approach.R
#
# compares two potential approaches for representing engineering compliance
# cost estimates in the model:
#   1.) a productivity shock
#   2.) an explicit abatement requirement per unit of output
# where the difference between the approaches is that under the productivity
# shock there is the opportunity to substitute away from the less productive
# input whereas under the explicit abatement requirement the sector is required
# to use a specified set of inputs for abatement per unit output without the
# ability to change the input requirements for abatement. under the productivity
# shock any substitution possibilities are based on the regulated sectors
# substitution elasticities.
#
# to simplify the comparison, in both scenarios it is assumed that there is no
# productivity growth in terms of the compliance activity.
#
# the hypothetical regulation is assumed to effect the primary metal
# manufacturing sector and have an engineering cost estiamte that compliance
# will require an additional 0.4% of baseline capital expenditures and 2% of
# labor expenditures. this is a simplified scenario to demonstrate the
# differences in approaches.
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# options
# ------------------------------------------------------------------------------

# use the default aggregation benchmark file
benchmark_file = "default_aggregation.gdx"

# sector being regulated in the hypothetical scenario
regulated_sector = "pmm"

# engineering estimates of capital and labor costs as share of baseline
# expenditures
capital_cost = .004
labor_cost = .02

# file containing the sample policy scenario
policy_file = "sample_policy.gms"



# ------------------------------------------------------------------------------
# function to write the policy file
# ------------------------------------------------------------------------------

write.policy = function(affected.sources,policy.type) {

  fc = file(file.path("examples",policy_file),open="wt")

  writeLines("set t_reg(t);",con=fc)
  writeLines("t_reg(t) = yes;",con=fc)
  writeLines("t_reg(tfirst) = no;",con=fc)

  if (policy.type=="abatement") {

    if (affected.sources=="new" | affected.sources=="all") {
      writeLines(paste0('ld_abate0(t_reg,r,"',regulated_sector,'","new") = ',
                        labor_cost,'*ld0(r,"',regulated_sector,
                        '")*l_prod_agg(t_reg);'),
                 con=fc)
      writeLines(paste0('kd_abate0(t_reg,r,"',regulated_sector,'","new") = ',
                        capital_cost,'*kd0(r,"',regulated_sector,'");'),
                 con=fc)
    }

    if (affected.sources=="existing" | affected.sources=="all") {
      writeLines(paste0('ld_abate0(t_reg,r,"',regulated_sector,'","extant") = ',
                        labor_cost,'*ld0(r,"',regulated_sector,
                        '")*l_prod_agg(t_reg);'),
                 con=fc)
      writeLines(paste0('kd_abate0(t_reg,r,"',regulated_sector,'","extant") = ',
                        capital_cost,'*kd0(r,"',regulated_sector,'");'),
                 con=fc)
    }

  } else if (policy.type=="productivity") {

    # to ensure that the productivity shock is of the same magnitude as the
    # abatement requirement we have to account for the changing cost shares in
    # the baseline
    if (affected.sources=="new" | affected.sources=="all") {
      writeLines(paste0('prod_ind(t_reg,r,"l","',regulated_sector,'","new") = ',
                        'prod_ind(t_reg,r,"l","',regulated_sector,'","new")+',
                        labor_cost,'*ld0(r,"',regulated_sector,
                        '")/ld_base(t_reg,r,"',regulated_sector,'");'),con=fc)
      writeLines(paste0('prod_ind(t_reg,r,"k","',regulated_sector,'","new") = 1+',
                        capital_cost,'*kd0(r,"',regulated_sector,
                        '")/kd_base(t_reg,r,"',regulated_sector,'");'),con=fc)
    }

    if (affected.sources=="existing" | affected.sources=="all") {
      writeLines(paste0('prod_ind(t_reg,r,"l","',regulated_sector,
                        '","extant") = prod_ind(t_reg,r,"l","',regulated_sector,
                                          '","extant")+',labor_cost,';'),con=fc)
      writeLines(paste0('prod_ind(t_reg,r,"k","',regulated_sector,
                        '","extant") = ',1+capital_cost,';'),con=fc)
    }

  }

  close(fc)

}



# ------------------------------------------------------------------------------
# run the baseline
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="baseline.csv",benchmark_file=benchmark_file)

# load the results for the baseline
bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)



# ------------------------------------------------------------------------------
# run regulation as a productivity shock
# ------------------------------------------------------------------------------

# run the policy scenario
write.policy("new","productivity")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
policy.prod = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
policy.prod$Sources = "New"

# run the policy scenario
write.policy("existing","productivity")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
temp = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
temp$Sources = "Existing"
policy.prod = rbind(policy.prod,temp)

# run the policy scenario
write.policy("all","productivity")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
temp = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
temp$Sources = "All"
policy.prod = rbind(policy.prod,temp)



# ------------------------------------------------------------------------------
# run regulation as an explicit abatement requirement
# ------------------------------------------------------------------------------

# run the policy scenario
write.policy("new","abatement")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
policy.abate = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
policy.abate$Sources = "New"

# run the policy scenario
write.policy("existing","abatement")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
temp = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
temp$Sources = "Existing"
policy.abate = rbind(policy.abate,temp)

# run the policy scenario
write.policy("all","abatement")
run.model(gdx_baseline_file="baseline.gdx",output_file="policy.csv",
          policy_file=paste0("examples",get.path.sep(),policy_file),
          benchmark_file=benchmark_file)

# load the results file
temp = read.csv(file.path("output","policy.csv"),stringsAsFactors=FALSE)
temp$Sources = "All"
policy.abate = rbind(policy.abate,temp)

# clean up policy file
file.remove(file.path("examples",policy_file))



# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# output equivalent variation
ev.prod.tot = aggregate(policy.prod$value[policy.prod$parameter=="ev_inf"],
                        by=list(policy.prod$Sources[policy.prod$parameter=="ev_inf"]),
                        sum)
names(ev.prod.tot) = c("Sources","value")
ev.abate.tot = aggregate(policy.abate$value[policy.abate$parameter=="ev_inf"],
                         by=list(policy.abate$Sources[policy.abate$parameter=="ev_inf"]),
                         sum)
names(ev.abate.tot) = c("Sources","value")
printf('    Sources Productivity  Abatement')
for (i in 1:nrow(ev.prod.tot))
  printf('%11s %12.2f  %9.2f',ev.prod.tot$Sources[i],ev.prod.tot$value[i],
                              ev.abate.tot$value[i])

# function to get the percentage change in inputs to the regulated sector
get.demand = function(var.name,include_extant=TRUE,include_new=TRUE) {

  data = NULL

  for (i in unique(policy.prod$Sources)) {

    # complete the variable name to match the results file
    var.name.ex = paste0(var.name,"_ex")

    # get the variable for the productivity shock case
    temp = policy.prod[policy.prod$parameter==var.name &
                       policy.prod$sector==regulated_sector &
                       policy.prod$Sources==i,]

    if (!include_new)
      temp$value = 0

    # add the demand assocaited with extant capital based production
    if (include_extant)
      temp$value = temp$value+
                   policy.prod$value[policy.prod$parameter==var.name.ex &
                                     policy.prod$sector==regulated_sector &
                                     policy.prod$Sources==i]

    # add the baseline value for converting to a percent change later
    temp$bau = bau$value[bau$parameter==var.name & bau$sector==regulated_sector]

    if (!include_new)
        temp$bau = 0

    # add the demand assocaited with baseline extant capital production
    if (include_extant)
      temp$bau = temp$bau+bau$value[bau$parameter==var.name.ex &
                                    bau$sector==regulated_sector]

    # label the version of the shock and store to be combined with other versions
    temp$Version = "Productivity"
    data = rbind(data,temp)

    # get the variable for the abatement shock case
    temp = policy.abate[policy.abate$parameter==var.name &
                        policy.abate$sector==regulated_sector &
                        policy.abate$Sources==i,]

    if (!include_new)
      temp$value = 0

    # add the demand assocaited with extant capital based production
    if (include_extant)
      temp$value = temp$value+
                   policy.abate$value[policy.abate$parameter==var.name.ex &
                                     policy.abate$sector==regulated_sector &
                                     policy.abate$Sources==i]

    # add in the sectoral demand for input due to abatement
    if (include_new & var.name %in% c("ld","kd","id"))
      temp$value = temp$value+
                   policy.abate$value[policy.abate$parameter==paste0(var.name,
                                                                     "_abate") &
                                      policy.abate$sector==regulated_sector &
                                      policy.abate$Sources==i]

    # add in the sectoral demand for input due to abatement
    if (include_extant & var.name %in% c("ld","kd","id"))
      temp$value = temp$value+
                   policy.abate$value[policy.abate$parameter==paste0(var.name,
                                                                     "_abate_ex") &
                                      policy.abate$sector==regulated_sector &
                                      policy.abate$Sources==i]

    # add the baseline value for converting to a percent change later
    temp$bau = bau$value[bau$parameter==var.name & bau$sector==regulated_sector]

    if (!include_new)
      temp$bau = 0

    # add the demand assocaited with baseline extant capital production
    if (include_extant)
      temp$bau = temp$bau+bau$value[bau$parameter==var.name.ex &
                                    bau$sector==regulated_sector]

    # label the version of the shock and combine with other versions
    temp$Version = "Abatement"
    data = rbind(data,temp)

  }

  # aggregate the variable across regions by year and version
  data = aggregate(data[,c("value","bau")],by=list(data$time,data$Version,
                                                   data$Sources),sum)
  names(data) = c("Year","Version","Sources","value","bau")

  # convert to a percentage change from the baseline
  data$value = (data$value/data$bau-1)*100

  return(data)

}

# plot the change in capital demand across the two cases
p1 = ggplot(get.demand("y"))+
       geom_hline(aes(yintercept=0))+
       geom_line(aes(x=Year,y=value,color=Version,linetype=Sources),size=1)+
       labs(y="Change in Output [%]")+
       scale_color_manual(values=sage.palette)+
       scale_linetype_manual(values=c("solid","dashed","dotted"))+
       theme(axis.line.x      = element_line(colour="black"),
             axis.line.y      = element_line(colour="black"),
             axis.ticks.y     = element_blank(),
             panel.grid.major = element_blank(),
             panel.grid.minor = element_blank(),
             panel.border     = element_blank(),
             panel.background = element_blank(),
             legend.key       = element_blank(),
             legend.key.width = unit(1.25,"cm"))
print(p1)

# plot the change in labor demand across the two cases
p2 = ggplot(get.demand("ld"))+
       geom_hline(aes(yintercept=0))+
       geom_line(aes(x=Year,y=value,color=Version,linetype=Sources),size=1)+
       labs(y="Change in Labor Demand [%]")+
       scale_color_manual(values=sage.palette)+
       scale_linetype_manual(values=c("solid","dashed","dotted"))+
       theme(axis.line.x      = element_line(colour="black"),
             axis.line.y      = element_line(colour="black"),
             axis.ticks.y     = element_blank(),
             panel.grid.major = element_blank(),
             panel.grid.minor = element_blank(),
             panel.border     = element_blank(),
             panel.background = element_blank(),
             legend.key       = element_blank(),
             legend.key.width = unit(1.25,"cm"))
print(p2)
