# scenario_analysis.R
#
# runs simulations similar to those in the paper
#
#  Marten, A., Garbaccio, R., and Wolverton, A. 2019. "Exploring the General
#  Equilibrium Costs of Sector-Specific Environmental Regulations." Journal of
#  the Association of Environmental and Resource Economists.
#
# the results will not be the sames as Marten et al. (2019) due to differences
# in the model version and the specific details of the simulations. code to
# replicate the specific simulations in Marten et al. (2019) is available on
# jaere's dataverse site at https://doi.org/10.7910/DVN/GOB7NW
#
# in the simulations below an abatement requirement is run for each sector
# starting in the model's second time period and the social costs for those
# hypothetical regulations are ploted as ratios relative to engineering cost
# estimates of such hypothetical abatement requirements. sensitivity analyses
# are conduceted over the input bias and size of the productivity shock along
# with key model parameters. outside of the simualtions that explicitly vary the
# input bias, the input bias is assumed to follow the nestor and pasurka input
# shares for pollution abatement activities.
#
# takes 30-120 minutes to run depending on processor speed
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# set options
# ------------------------------------------------------------------------------

# benchmark file
benchmark_file = "default_aggregation.gdx"

# mapping file associated with the benchmark
map.file = "default_aggregation.gms"

# shock size - billions of benchmark year dollars
shock.size = 0.1

# shock type for sensitivity analyses
sensitivity.shock = "np.air"

# output directory to store plots
image.dir = file.path("output","scenario_analysis")



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# set global variables
# ------------------------------------------------------------------------------

# get the list of sectors
sectors = get.sage.sectors(map.file)



# ------------------------------------------------------------------------------
# function definitions
# ------------------------------------------------------------------------------

# function to write the shock definition file for use with the policy file
# examples/productivity_shocks.gms
write.productivity.shock = function(shock.type,shock.size,sector,ex_share=-1) {

  # determine the appropriate flag for the shock type
  if (shock.type=="hicks")
    type = 1
  else if (shock.type=="np.air")
    type = 2
  else if (shock.type=="capital")
    type = 3
  else if (shock.type=="labor")
    type = 4

  # create a gams input file containing the policy shock
  fc = file(file.path("examples","shock_definition.gms"),open="wt")
  writeLines(paste0('s_reg(\"',sector,'\") = YES;'),con=fc)
  writeLines(paste0('ex_shock_share = ',ex_share,';'),con=fc)
  writeLines(paste0('shock_type = ',type,';'),con=fc)
  writeLines(paste0('shock_size = ',shock.size,';'),con=fc)
  writeLines('',con=fc)
  close(fc)

}



# this function loops through each sector writes a policy file for the requested
# shock, runs the model, and stores the welfare change, which is then returned
run.experiments = function(shock.type,options="",shock.size=0.1,
                           benchmark_file="default_aggregation.gdx",
                           parameter_file="parameters.gms",
                           starting_values=NA,
                           baseline_file="baseline.gms") {

  # run the baseline
  if (is.na(starting_values))
    run.model(benchmark_file=benchmark_file,output_file="baseline.csv",
              options=options,
              parameter_file=parameter_file,
              baseline_file=baseline_file)
  else
    run.model(benchmark_file=benchmark_file,output_file="baseline.csv",
              options=options,gdx_baseline_file=starting_values,
              parameter_file=parameter_file,
              baseline_file=baseline_file)
  bau = read.csv(file.path("output","baseline.csv"),stringsAsFactors=FALSE)

  # name of output file with the model results
  output_file = "cost_experiments.csv"

  # print a header denoting the start of a round of experiments
  print("")
  print(paste0("running ",shock.type,"..."))

  # initialize container for the results
  results = NULL

  # loop over each sector in the model and shock it
  for (i in 1:length(sectors)) {

    # current sector
    sector = sectors[i]

    # write the policy file for the sector
    sector.shock.size = write.productivity.shock(shock.type,
                                                 shock.size,
                                                 sector,
                                                 ex_share=-1)

    # run the policy scenario
    # note that "sector" is not a real option for the model, if there is an
    # error in the solve r prints out a warning message with the system call
    # and this way we can identify which sector had the error
    print(paste0("running ge model for ",sector,"..."))
    run.model(benchmark_file=benchmark_file,
              output_file=output_file,
              policy_file="examples/abatement_requirement.gms",
              parameter_file=parameter_file,
              gdx_baseline_file="baseline.gdx",
              gdx_save=0,
              options=paste(options," --sector=",sector),
              baseline_file=baseline_file)

    # pause to allow gams to fully release the csv file with the results
    #Sys.sleep(1.5)

    # load the results
    ge.policy = read.csv(file.path("output",output_file),stringsAsFactors=FALSE)

    # extract the welfare change
    temp = ge.policy[ge.policy$parameter=="ev",c("region","household","value")]

    # save the aggregate welfare change
    temp = data.frame(sector=sector,value=sum(temp$value),
                      stringsAsFactors=FALSE)

    temp$sector = sector

    # calculate the engineering cost estimate consistent with the current shock
    y = combine.new.extant(bau,"y",sector=sector)
    y0 = y[y$time==min(y$time),]
    names(y0)[names(y0)=="value"] = "y0"
    y = merge(y,y0[,names(y0)!="time"],all.x=TRUE)
    y = aggregate(y[,c("value","y0")],by=list(y$time),sum)
    names(y) = c("time","value","y0")
    y$value = y$value/y$y0*1/1.045^(y$time-min(y$time))*shock.size

    # if not a static run remove the first period from the engineering costs
    # since there is not shock in that period
    if (length(unique(y$time))>1)
      y$value = y$value*(y$time!=min(y$time))

    temp$eng = sum(y$value)

    results = rbind(results,temp)

  }

  # add the shock name to the results
  results$shock = shock

  return(results)

}



# function to conduct a sensitivity analysis that is defined by supply either
# lines to place in a prologue file, command line options, or a different
# parameter file
run.sensitivity = function(prologue,scenario,options="",
                           parameter_file="parameters.gms",
                           starting_values=NA,
                           baseline_file="baseline.gms") {

  # create a prologue file with the sensitivity values
  prologue_file = "sensitivity_prologue.gms"
  fc = file(file.path("examples",prologue_file))
  writeLines(prologue,con=fc)
  close(fc)

  # run the experiments
  temp = run.experiments(sensitivity.shock,
                         shock.size=shock.size,
                         options=paste0(options,
                                        " --prologue=examples",
                                        get.path.sep(),
                                        prologue_file),
                         parameter_file=parameter_file,
                         starting_values=starting_values,
                         baseline_file=baseline_file)
  temp$scenario = scenario

  return(temp)

}



# function to plot the results in the form of social costs percentage deviation
# from engineering cost estiamtes
plot.results = function(data,limits=NA,label=NULL,scenario.order=NA) {

  # calculate the percentage deviation of social costs from engineering costs
  data$value = (data$value/data$eng-1)*100

  # make sure the scenarios are discrete variables
  if (any(!is.character(data$scenario)))
    data$scenario = as.character(data$scenario)

  # get the min/max rates for the crossbars on the plot
  bars = aggregate(data$value,by=list(data$scenario),min)
  names(bars) = c("scenario","min")
  bars$max = aggregate(data$value,by=list(data$scenario),max)$x
  bars$mean = aggregate(data$value,by=list(data$scenario),mean)$x

  # jitter for the sector names to seperate vertically in the plot
  set.seed(202)
  width = .4
  jitter = data.frame(sector=unique(data$sector),
                      jitter=runif(length(unique(data$sector)))*2*width-width)
  data = merge(data,jitter,sort=FALSE)

  # set the order of the scenarios if provided
  if (any(!is.na(scenario.order))) {
    data$scenario = factor(data$scenario,rev(scenario.order))
    bars$scenario = factor(bars$scenario,rev(scenario.order))
  }

  p = ggplot()+
        geom_crossbar(data=bars,aes(x=scenario,y=mean,ymin=min,ymax=max),
                      width=2*width*1.1,color="grey")+
        geom_text(data=data,aes(x=scenario,y=value,label=sector),
                  nudge_x=data$jitter,
                  size=2.5)+
        coord_flip()+
        labs(y="% Difference between Equivalent Variation and Engineering Costs",
             x=label,
             title=paste0("SAGE ",get.sage.version()))+
        guides(color=FALSE)+
        theme(axis.line.x      = element_line(colour="black"),
              axis.line.y      = element_line(colour="black"),
              axis.ticks.y     = element_blank(),
              panel.grid.major = element_blank(),
              panel.grid.minor = element_blank(),
              panel.border     = element_blank(),
              panel.background = element_blank(),
              legend.key       = element_blank(),
              legend.position  = c(.89,.3))

  if (!any(is.na(limits)))
    p = p+scale_y_continuous(limits=limits)

  print(p)

}



# ------------------------------------------------------------------------------
# run the ge experiments
# ------------------------------------------------------------------------------

# run the default model baseline to use as starting values
run.model(benchmark_file=benchmark_file,output_file="baseline_default.csv")

# container for the results
results = NULL

# shock types to run
shock.types = list("hicks","capital","labor","np.air")

# run the experiments
for (shock in shock.types)
  results = rbind(results, run.experiments(shock,shock.size=shock.size,
                                           benchmark_file=benchmark_file,
                                           starting_values="baseline_default.gdx"))



# ------------------------------------------------------------------------------
# run the shock size sensitivities
# ------------------------------------------------------------------------------

# values for the shocks to be included in the sensitivity analysis
shock.sizes = c(.4,.8,1.2,1.6,2)

# container for the results intialized with the default results
results.size = results[results$shock==sensitivity.shock,]
results.size$scenario = shock.size

for (s.size in shock.sizes) {

  # run the experiments
  temp = run.experiments(sensitivity.shock,shock.size=s.size,
                         starting_values="baseline_default.gdx")

  # convert to percentage difference from engineering costs
  temp$scenario = s.size

  # save the results
  results.size = rbind(results.size,temp)

}



# ------------------------------------------------------------------------------
# setup the sensitivity analyses
# ------------------------------------------------------------------------------

# container for the sensitivity analysis results
results.sen = results[results$shock==sensitivity.shock,]
results.sen$scenario = "Default"



# ------------------------------------------------------------------------------
# run the labor supply sensitivities
# ------------------------------------------------------------------------------

# perfectly inelastic labor supply
results.sen = rbind(results.sen,
                    run.sensitivity(c("lse_sub = 0.000001;",
                                      "lse_inc = -0.000001;"),
                                    "Perfectly Inelastic Labor Supply",
                                    starting_values="baseline_default.gdx"))

# high labor supply elasticity
results.sen = rbind(results.sen,
                    run.sensitivity(c("lse_sub = 0.4;"),
                                    "High Labor Supply Elasticity",
                                    starting_values="baseline_default.gdx"))



# ------------------------------------------------------------------------------
# run the value-added substitution elasticity sensitivities
# ------------------------------------------------------------------------------

# low capital-labor elasticity
results.sen = rbind(results.sen,
                    run.sensitivity(c("se_kl(s) = .5*se_kl(s);"),
                                    "Low KL Elasticity",
                                    starting_values="baseline_default.gdx"))

# high capital-labor elasticity
results.sen = rbind(results.sen,
                    run.sensitivity(c("se_kl(s) = 1.5*se_kl(s);"),
                                    "High KL Elasticity",
                                    starting_values="baseline_default.gdx"))

# unit capital-labor elasticity
results.sen = rbind(results.sen,
                    run.sensitivity(c("se_kl(s) = 1.0001;"),
                                    "Unit KL Elasticity",
                                    starting_values="baseline_default.gdx"))



# ------------------------------------------------------------------------------
# run static vs. dynamic comparison
# ------------------------------------------------------------------------------

# static sensitivity
results.sen = rbind(results.sen,
                    run.sensitivity("",
                                    "Static",
                                    baseline_file="baseline_static.gms"))



# ------------------------------------------------------------------------------
# plot the results
# ------------------------------------------------------------------------------

# create output directory if it doesn't exist
if (!dir.exists(image.dir))
  dir.create(image.dir)

# rename the input composition scenarios
names(results)[names(results)=="shock"] = "scenario"
results$scenario[results$scenario=="hicks"] = "Neutral"
results$scenario[results$scenario=="capital"] = "Capital Only"
results$scenario[results$scenario=="labor"] = "Labor Only"
results$scenario[results$scenario=="np.air"] = "Nestor & Pasurka"

# reverse sign of EV to be comparable to cost measure
results$value = -results$value
results.sen$value = -results.sen$value
results.size$value = -results.size$value

plot.results(results,limits=c(-15,40))
ggsave(file.path(image.dir,"scenario_analysis_composition.png"),device="png")

# order for plotting the sensitivity analyses
scenario.order =  c("Default",
                    "Perfectly Inelastic Labor Supply",
                    "High Labor Supply Elasticity",
                    "Low KL Elasticity",
                    "High KL Elasticity",
                    "Unit KL Elasticity",
                    "Static")

plot.results(results.sen,limits=c(-15,40),scenario.order=scenario.order)
ggsave(file.path(image.dir,"scenario_analysis_sensitivities.png"),device="png")

plot.results(results.size,limits=c(-15,40),
             label="Engineering Costs in First Year [Billion $]")
ggsave(file.path(image.dir,"scenario_analysis_size.png"),device="png")



# ------------------------------------------------------------------------------
# cleanup files
# ------------------------------------------------------------------------------

file.remove(file.path("examples","shock_definition.gms"))
file.remove(file.path("examples","sensitivity_prologue.gms"))
