# static_vs_dynamic.R
#
# compare the outputs from the static and dynamic models
#
# uses the sample regulation in examples/sample_abatement_requirement.gms
#
# this example is intended to be run from the top level directory of the model
#



# ------------------------------------------------------------------------------
# load libraries
# ------------------------------------------------------------------------------

# load in the R utilities for sage
source(file.path("utilities","R_utilities.R"))

library(ggplot2)



# ------------------------------------------------------------------------------
# run the dynamic model
# ------------------------------------------------------------------------------

# run the baseline
run.model(output_file="dynamic_baseline.csv")

# run the policy scenario
run.model(output_file="dynamic_policy.csv",
          gdx_baseline_file="dynamic_baseline.gdx",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))



# ------------------------------------------------------------------------------
# run the static model
# ------------------------------------------------------------------------------

# run the baseline
run.model(baseline_file="baseline_static.gms",
          output_file="static_baseline.csv")

# run the policy scenario
run.model(baseline_file="baseline_static.gms",
          gdx_baseline_file="static_baseline.gdx",
          output_file="static_policy.csv",
          policy_file=paste0("examples",get.path.sep(),
                             "sample_abatement_requirement.gms"))


# ------------------------------------------------------------------------------
# display the results
# ------------------------------------------------------------------------------

# load the results
dynamic.bau    = read.csv(file.path("output","dynamic_baseline.csv"),
                          stringsAsFactors=FALSE)
dynamic.policy = read.csv(file.path("output","dynamic_policy.csv"),
                          stringsAsFactors=FALSE)
static.bau     = read.csv(file.path("output","static_baseline.csv"),
                          stringsAsFactors=FALSE)
static.policy  = read.csv(file.path("output","static_policy.csv"),
                          stringsAsFactors=FALSE)

# set the sector to plot
plot.sector = "pmm"

# variable to plot
plot.variable = "y"

# get the results under the putty-putty assumption
temp_d = combine.new.extant(dynamic.bau,plot.variable,plot.sector)
temp_d$policy = combine.new.extant(dynamic.policy,plot.variable,
                                   plot.sector)$value
temp_d$scenario = "Dynamic"

# get the results under the putty-clay assumption
temp_s = temp_d
temp_s$value = combine.new.extant(static.bau,plot.variable,plot.sector)$value
temp_s$policy = combine.new.extant(static.policy,plot.variable,
                                   plot.sector)$value
temp_s$scenario = "Static"

# calcualte the percentage change in the national aggregate value
plot_data = rbind(temp_d,temp_s)
plot_data = aggregate(plot_data[,c("value","policy")],
                      by=list(plot_data$time,plot_data$scenario),sum)
names(plot_data) = c("time","scenario","value","policy")
plot_data$value = (plot_data$policy/plot_data$value-1)*100

# plot the variable change
p = ggplot(plot_data)+
      geom_line(aes(x=time,y=value,linetype=scenario),size=1.1)+
      labs(x="Year",y=paste0("% Change in ",plot.variable," in ",plot.sector))+
      guides(linetype=guide_legend(title="Scenario"))+
      theme(axis.line.x      = element_line(colour="black"),
            axis.line.y      = element_line(colour="black"),
            axis.ticks.y     = element_blank(),
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.border     = element_blank(),
            panel.background = element_blank(),
            legend.key       = element_blank())

print(p)

# output compensating variation
ev = sum(dynamic.policy$value[dynamic.policy$parameter=="ev_inf"])
printf('Annualized EV dynamic..................: %7.2f',ev*.045)
ev = sum(as.numeric(static.policy$value[static.policy$parameter=="ev"]))
printf('Annualized EV static...................: %7.2f',ev)
